%% GRAM_FACTOR
% Define a grammaticality factor to distinguish between
% grammatical and non-grammatical test items.

%% Basic grammaticality factor
%
% GRAM_FACTOR(aglss, 'fname') distinguishes between grammatical (G) and
% non-grammatical (NG) test items.
%
% The example below generates training items based on the XY_GRAMMAR, which
% alternates Xs and Ys, and test items that are either grammatical
% (strictly alternating) or nongrammatical (with repeated Xs or Ys). 

s_xy = aglss(xy_grammar, [1 10]);

[s, levs] = gram_factor(s_xy, 'gram');

s = factorial_testsets(s, [{'gram'}, levs]);
s = choose_items(s, 4, 2);

disp('Training items:');
disp(format_train_items(s));

disp('Test items:');
disp(format_test_items(s));

%% Grammaticality factor with specified category names
%
% GRAM_FACTOR(aglss, 'fname', {'gname', 'ngname'}) specifies alternative
% names for the grammatical and nongrammatical categories.

s = gram_factor(s_xy, 'MyGram', {'OK', 'NOK'});

s = factorial_testsets(s, {'MyGram', 'OK', 'NOK'} );
s = choose_items(s, 4, 2);

disp('Training items:');
disp(format_train_items(s));

disp('Test items:');
disp(format_test_items(s));

%% Grammaticality factor with specified grammar
%
% GRAM_FACTOR(aglss, 'fname', fsg) distinguishes test items that are
% grammatical (G) or non-grammatical (NG) with respect to a specified
% grammar.
%
% The example below generates training items based on the XY_GRAMMAR, and
% test items that are either grammatical or non-grammatical with respect to
% this grammar, with the constraint that all test items are consistent with
% the XX_GRAMMAR.  This allows NG items with repeated Xs, but never
% repeated Ys.

s_xy = aglss(xy_grammar, [1 10]);

[s, levs] = gram_factor(s_xy, 'gram');  % grammaticality w.r.t. training item grammar

[s, levs] = gram_factor(s, 'xx_gram', xx_grammar);   % grammaticality w.r.t. more liberal grammar

s = factorial_testsets(s, [{'gram'}, levs], {'xx_gram', 'G'});
s = choose_items(s, 4, 2);

disp('Training items:');
disp(format_train_items(s));

disp('Test items:');
disp(format_test_items(s));
