%% Prior stimuli
% Compute characteristics of extant training and test sets, such as those
% from previous studies.
%
% In addition to its ability to select sets of AGL training and test items
% that manipulate various characteristics, StimSelect also provides
% functions to evaluate the same characteristics of specified training and
% test sets.  These functions require training and test items to be
% listed in character arrays or cell arrays of strings.  Some of the
% functions also require a finite state grammar to be defined.

%% Importing a list of training items
%
% To evaluate StimSelect characteristics of a specified AGL stimulus set,
% the training items must be listed as strings in a character array or a
% cell array.  Often, the first step is to enter the set of training items
% into a text file (e.g. my_train.txt), with one item per line.  
%
% Once the file of training items has been created, the Import
% Wizard can be used to load the items into a corresponding
% variable in Matlab.  Select Import Data from the File menu, browse
% to the appropriate file, and away you go.  Sample data can be found in
% pb_train.txt, which is located in the Sample_Scripts directory of
% StimSelect.
%
% Alternatively, the textscan() function can be used, as shown below.
%
% Matlab provides many ways to load data from files or the clipboard.  See
% the Data Import and Export chapter of the Matlab programming manual for
% details.

fid = fopen('pb_train.txt');
C = textscan(fid, '%s');
fclose(fid);
pb_train = C{1};

pb_train

%% Importing a list of test items
% As with training items discussed above, a set of test items can be
% entered into a text file (e.g. my_test.txt), and the Import Wizard can be
% used to load the items into a Matlab variable.  
% Sample test items can be found in pb_test.txt, which is located in the
% Sample_Scripts directory of StimSelect.
%
% Alternatively, the
% textscan() function can be used, as shown below.

fid = fopen('pb_test.txt');
C = textscan(fid, '%s');
fclose(fid);
pb_test = C{1};

pb_test

%% Remove trailing blanks if necessary
% If the strings containing the training and test items include leading or 
% trailing blanks, they should be removed using the strtrim() function.

pb_train = strtrim(pb_train);
pb_test = strtrim(pb_test);

%% Compute frequency of chunks in training items
% The TRAINSET_CHFREQ function computes the frequencies of training item
% chunks (sub-strings).  See *_help trainset_chfreq_* for
% details.
%
% Q = TRAINSET_CHFREQ(train_items, max_size) returns the frequencies for
% chunks of size 1 to MAX_SIZE.  The return value is cell array.
% Each Q{i} contains a matrix of frequency counts for chunks of size i.
%
% [Q, S, CX] = TRAINSET_CHFREQ(...) also returns a string S listing the
% symbols from which the training items are composed, and a cell array
% CX in which each cell CX{i} is a cell array of strings listing the
% possible chunks of length i.

max_chsize = 3; % trigrams are big enough
[chfreq, syms, chunks] = trainset_chfreq(pb_train, max_chsize);

disp(' ');
disp(' chunk freq');
for sz = 1:max_chsize,
    if sz <= 2,
        ch = chunks{sz};
        fq = chfreq{sz};
        for i = 1:numel(fq),
            fprintf(1, '   %-2s   %2d\n', ch(i,:), fq(i));
        end
    else
        disp('Larger chunks computed but not shown in this example'),
    end
end

%% Identify G and NG items
% Test items can be identified as grammatical (G) or non-grammatical (NG)
% by attempting to parse them with a specified finite state grammar.  
%
% ISGRAMMATICAL(fsg, test_items) returns a vector of 1's and 0's
% indicating which of the specified test items are G (1) and which are NG
% (0), relative to the specified grammar (FSG).
%
% See <StimSelect_summary.html> for information on how to specify a
% grammar.
%
% The examples below use the grammar from Knowlton & Squire (1996, Exp. 1),
% which is pre-defined in a function included with StimSelect.

f = knowlton_squire_grammar();
isg = isgrammatical(f, pb_test);

disp(' ');
disp('item       gram');
for i = 1:length(pb_test),
    fprintf(1, '%-10s ', pb_test{i});
    if isg(i)
        fprintf(1, 'G');
    else
        fprintf(1, 'NG');
    end
    fprintf(1, '\n');
end

%% Determine similarity between test and training items
% The TESTSET_SIM function computes whole-item similarities
% between test and training items.
%
% TESTSET_SIM(test_items, train_items) returns the average similarity of each
% test item to the whole set of training items.  See *_help testset_sim_* for
% more options.

sim = testset_sim(pb_test, pb_train);

disp(' ');
disp('item        sim');
for i = 1:length(pb_test),
    fprintf(1, '%-10s  %4.3f\n', pb_test{i}, sim(i));
end

%% Compute novel chunks in test items
% The TESTSET_CHNOV function computes chunk novelty for a set of test items
% relative to specified training items.
%
% TESTSET_CHNOV(test_items, train_items, sz) returns the number of chunks
% of length SZ in each test item that do not appear in any of the specified
% training items.

chsize = 2;
chnov = testset_chnov(pb_test, pb_train, chsize);

disp(' ');
disp('item        chnov');
for i = 1:length(pb_test),
    fprintf(1, '%-10s  %d\n', pb_test{i}, chnov(i));
end

%% Compute chunk strength
% The TESTSET_CHSTR function computes chunk strength for a set of test items
% relative to specified training items.  It is ordinarily desirable to
% specify a particular grammar so that chunk strength can be normalized by
% the number of grammatical chunks.
%
% TESTSET_CHSTR(test_items, train_items, fsg, sz) returns the chunk
% strength for each test item, considering chunks of length 2 to SZ.
% Chunk strength is normalized by the number of
% chunks allowed by grammar FSG.

max_chsize = 3;
f = knowlton_squire_grammar();
chstr = testset_chstr(pb_test, pb_train, f, max_chsize);

disp(' ');
disp('item        chstr');
for i = 1:length(pb_test),
    fprintf(1, '%-10s  %4.3f\n', pb_test{i}, chstr(i));
end

%% Compute rule strength
% The TESTSET_RULESTR function computes rule strength for a set of test items
% relative to specified training items.
%
% TESTSET_RULESTR(test_items, train_items, [], sz) returns the rule
% strength of each test item based on the fraction of chunks of length 2 to
% SZ in the test item that also appear in at least one training item.

max_chsize = 3;
f = knowlton_squire_grammar();
rulestr = testset_rulestr(pb_test, pb_train, [], max_chsize);

disp(' ');
disp('item        rulestr');
for i = 1:length(pb_test),
    fprintf(1, '%-10s  %4.3f\n', pb_test{i}, rulestr(i));
end
