function f = fsg( arg1, arg2, arg3 )
% FSG       Make a finite state grammar object
%
% f = fsg()
% f = fsg( links )
% f = fsg( links, symbols )
% f = fsg( [], symbols )
% f = fsg( [], symbols, N )
% f = fsg( n_symbols )
% f = fsg( training, [@induce_fctn] )
%
% Parameters specify optional arguments:
%   links - matrix of states (rows) and symbol indices (cols) whose values
%           indicate next-state values (0..n_states), where 0 means no 
%           transition for that symbol; symbol index 1 is always for the
%           end-of-string symbol, which can link to state 0 if
%           end-of-string is not valid for a particular state, or to any
%           non-zero value if end-of-string is valid;
%           if links is not specified or [], the default is a single state 
%           with no transitions, and a single symbol (end-of-string)
%   symbols - optional string of characters, in order by symbol index, that
%             can label transitions between states;
%             default is to use abstract symbol indices only, 1..n_symbols
%   N - Number of states in the grammar
%   n_symbols - Max number of symbols, including end-of-string symbol
%   training - array of training strings for grammar induction
%   @induce_fctn - handle to function or method to use for grammar
%               induction
%
% Generally, an FSG is a mapping from SxC -> S', where S is a set of
% states, C is a set of symbols, and S' = S or -1.
%
% Methods for fsg objects:
%   fsg         - Make an fsg object
%   symbols     - Get/set list of symbol characters for transitions
%   n_symbols   - Get/set number of symbols in language
%   n_states    - Get number of states in fsg
%   n_links     - Get number of links in fsg
%   link        - Get/set link from one state to another for given symbol
%   add_state   - Add new state to fsg
%   delete_state
%
%   grammatical_strings     - Get grammatical strings or chunks
%   generate_sample_strings - Get sample of G and NG strings
%   upstream    - Get set of states upstream from given state
%
%   start       - Set current set of states to [1]
%   state       - Get/set current set of states (1..n)
%   parse_symbol - Attempt to parse symbol at current state
%   parse_string
%
%   find_link   - (obsolete?) Get first/next link between two states for given symbol

% Copyright 2007 Todd M. Bailey
%
% This file is part of AGL StimSelect (AGLSS).
%
%     AGLSS is free software; you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation; either version 3 of the License, or
%     (at your option) any later version.
% 
%     AGLSS is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with this program.  If not, see <http://www.gnu.org/licenses/>.

training = [];
induce_fctn = @induce1;     % Default function to induce grammar from training

% Default values
links = [];
symbols = '';
n_symbols = length(symbols) + 1;    % end-of-string is extra;
n_states = 0;

if nargin,
    if isa(arg1, 'fsg'),
        f = arg1;
        return
    end
    if ischar(arg1)
        % training strings
        training = arg1;
        if nargin > 1 && isa(arg2, 'function_handle'),
            induce_fctn = arg2;
        end
%         links = [];
        symbols = unique_chars(training);
%         n_symbols = length(symbols) + 1;    % end-of-string is extra
    elseif isscalar(arg1)
        n_symbols = arg1;
    else
        % must be array of state transitions
        links = arg1;
        n_states = size(links, 2);
        if nargin > 1,
            symbols = arg2;
            if nargin > 2,
                if isempty(links),
                    n_states = arg3;
                else
                    error('FSG:LinkStateConflict', ...
                        'Links and number of states not both allowed');
                end
            end
        else
            n_symbols = size(links, 2);
        end
    end
end

if symbols,
    n_symbols = length(symbols) + 1;    % end-of-string is extra;
end
    
if isempty(links),
    f.links = zeros(1, n_symbols);    % allocate room for end-of-string, too
else
    f.links = links;
end

f.symbols = symbols;

f.state = 1;    % Start state
f.prefix = {};  % String leading up to each state
f.known_strings = {};   % Will store observed substrings for each state, from training
f.strings = [];     % Grammatical strings (when computed)
f.string_bounds = [];   % Stores min and max length of stored grammatical strings
f.upstream = {};   % Placeholder for which states lead to which others

f = class(f, 'fsg');

if isempty(links) && n_states,
    for i = 1:n_states-1,
        [nil, f] = add_state(f);
    end
end

if training,
    f = induce_fctn(f, training);
end

return
