function [strings, isG] = generate_sample_strings(cur_f, minmaxlen, ...
    max_items)
% generate_sample_strings - Generate G and NG strings
%
% [S, isG] = generate_sample_strings(F, [minL maxL], maxI)
%  Generates grammatical (G) and ungrammatical (NG) strings using the
%  grammar specified in F, which should be an FSG object.  minL and maxL
%  specify the minimum and maximum string lengths. maxI specifies the
%  maximum number of combined grammatical and ungrammatical items.  If
%  necessary, random subsets will be chosen to limit the total.
%
% [S, isG] = generate_sample_strings(F, [minL maxL], [maxG maxNG])
%   maxG and maxNG specify the maximum number of G and NG items,
%   respectively. May be inf.
%
% The return value S will be a cell array of strings.
%
% The return value isG will be a logical vector indicating which strings S
% are grammatical.  S(isG) will select the grammatical cells, and S(~isG)
% will select the ungrammatical ones.
%
% SEE ALSO:
%  fsg, grammatical_strings

% Copyright 2013 Todd M. Bailey
%
% This file is part of AGL StimSelect (AGLSS).
%
%     AGLSS is free software; you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation; either version 3 of the License, or
%     (at your option) any later version.
% 
%     AGLSS is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with this program.  If not, see <http://www.gnu.org/licenses/>.

if nargin < 3,
    max_items = Inf;
end
if length(max_items) < 2,
    max_g = floor(max_items/2);
%     if isinf(max_g),
%         max_ng = inf;
%     else
%         max_ng = max_items - max_g;
%     end
else
    max_g = max_items(1);
%     max_ng = max_items(2);
%     max_items = max_g + max_ng;
end

ng_lim = 1E6;   % Maximum NG items to consider exhaustively at each length

syms = symbols(cur_f);  % Symbols used by the grammar
n_syms = length(syms);

fprintf(1, '\n Potential items:\n');
fprintf(1, '\tGrammar involves %d symbols (%s)\n', ...
    n_syms, syms);

% How many possible strings of each length?
if isempty(minmaxlen),
    min_len = 1;
else
    min_len = minmaxlen(1);
end
if length(minmaxlen) < 2,
    % determine max_len from string space
    if isinf(max_g),
        error('FSG:generate_sample:IndeterminateMaxLen', ...
            'Max string length or max items must be specified');
    end
    n_strings = [];
    max_len = min_len - 1;
    while sum(n_strings) < max_g,
        max_len = max_len + 1;
        n_strings(end+1) = n_syms .^ max_len;
    end
else
    max_len = minmaxlen(2);
    n_strings = repmat(n_syms, 1, max_len-min_len+1) .^ (min_len:max_len);
end
n_stotal = sum(n_strings); 
fprintf(1, '\t%d possible strings of length %d-%d\n', ...
    n_stotal, min_len, max_len);

% Generate all grammatical (G) items of relevant lengths
string_g = grammatical_strings(cur_f, [min_len, max_len]);
if ~isempty(string_g),
    string_g = cellstr(string_g);
end
n_g = length(string_g);
n_ng = n_stotal - n_g;
fprintf(1, '\t\t%d grammatical strings (%5.2f%%)\n', ...
    length(string_g), 100* n_g / (n_g + n_ng));

fprintf(1, '\t\t%d ungrammatical strings (%5.2f%%)\n', ...
    n_ng, 100 * n_ng / (n_g + n_ng));


% Compute upper bound on number of grammatical and ungrammatical items
% but assume there are many more potential NG items than G
max_g = min([n_g, max_g]);
if max_g < n_g,
    fprintf(1, '\tUsing sample of %d grammatical strings\n', ...
        max_g);
    string_g_samp = randrow(string_g, max_g);
else
    fprintf(1, '\tUsing all %d grammatical strings\n', ...
        n_g);
    string_g_samp = string_g;
end

if length(max_items) == 1,
    max_ng = max_items - max_g;
else
    max_ng = min([max_items(2), sum(max_items(1:2)) - max_g]);
end
max_ng = min([max_ng, n_stotal - n_g]);
if max_ng < n_ng,
    fprintf(1, '\tUsing sample of %d ungrammatical strings\n', ...
        max_ng);
else
    fprintf(1, '\tUsing all %d ungrammatical strings\n', ...
        n_ng);
end

% Generate ungrammatical (NG) items of relevant lengths
n2go = max_ng;
string_ng = cell(n2go, 1);
nxt_ng = 1;
for lenidx = 1:max_len-min_len+1,
    curlen = min_len+lenidx-1;
    % desired number of strings of this length
    ntgt = round(n2go * n_strings(lenidx) / sum(n_strings(lenidx:end)));
    
    if n_strings(lenidx) <= ng_lim || isinf(max_items),
        % Brute force, generate all the strings quickly and drop G ones
        ff = fullfact( repmat(n_syms, curlen, 1) );
        if isempty(string_g),
            cur_items = cellstr(syms(ff));
        else
            cur_items = setdiff(cellstr(syms(ff)), string_g);
        end
        if length(cur_items) > ntgt,
            % Keep random subset
            cur_items = randrow(cur_items, ntgt);
        end
    else
        % Too many for brute force
        % Generate strings at random and accumulate NG ones
        fprintf(1, '\t (looking for %d NG strings of length %d...)\n', ...
            ntgt, curlen);
    
        % Start with a manageable basis set of strings
        baselen = floor(log(ng_lim) / log(n_syms) - 1);
        basefact = fullfact( repmat(n_syms, baselen, 1) );
        sloplen = mod(curlen, baselen);
        if sloplen > 0,
            slopfact = fullfact( ...
                repmat(n_syms, sloplen, 1) );
        else
            slopfact = [];
        end

        cur_items = {};
        while length(cur_items) < ntgt,
            item = [];
            while size(item, 2) + size(basefact, 2) <= curlen,
                item = [item, randrow(basefact)];
            end
            if size(item, 2) < curlen,
                item = [item, randrow(slopfact)];
            end
            cur_string = syms(item);
            % Check if it is G or NG
            [states, nil, leftover] = parse_string(cur_f, cur_string);
            if ~isempty(leftover),
                % This is NG, keep it
                cur_items = union_sorted({cur_string}, cur_items);
            end
        end
    end
    n = length(cur_items);
    string_ng(nxt_ng:nxt_ng+n-1) = cur_items;
    nxt_ng = nxt_ng + n;
    n2go = n2go - n;
end

if n2go > 0,
    % Did not fill the quota, so remove empty place holders
    string_ng(nxt_ng:end) = [];
end
disp(' ');

% pool G and NG, and remember which are which
strings = [string_g_samp; string_ng];
isG = false(1,length(strings));   % allocate space
isG(1:length(string_g_samp)) = true;  % logical index of G vs. NG

end