function [gstrings, f] = grammatical_strings(f, bounds, curstates)
% GRAMMATICAL_STRINGS   Generate grammatical strings for a FSG
%
% gstrings = grammatical_strings(fsg, max_len)
% gstrings = grammatical_strings(fsg, [min_len, max_len])
% gstrings = grammatical_strings(fsg, ..., start_states)
%
%   Returns a character array of strings generated by the specified finite
%   state grammar.  The empty string, if grammatical, is returned as ".".
%   
%   MAX_LEN specifies the maximum length of strings to generate.
%   [MIN_LEN, MAX_LEN] specifies the minimum and maximum length strings to
%   generate.  The default value for MIN_LEN is 0.
%
%   START_STATES optionally specifies a vector of states from which to begin
%   generating chunks (not just fully-parseable strings).  To generate
%   initial chunks, specify START_STATES as [] or start_state(f).  The
%   special value -1 generates grammatical endings.
%       EXAMPLE:    GRAMMATICAL_STRINGS(f, [2 2], 1:n_states(f))
%           returns grammatical bigrams for grammar F.
%
%       EXAMPLE:    GRAMMATICAL_STRINGS(f, [2 2], start_state(f))
%           returns legal string-ending bigrams
%
%       EXAMPLE:    GRAMMATICAL_STRINGS(f, [2 2], -1)
%           returns legal string-ending bigrams

% Copyright 2007 Todd M. Bailey
%
% This file is part of AGL StimSelect (AGLSS).
%
%     AGLSS is free software; you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation; either version 3 of the License, or
%     (at your option) any later version.
% 
%     AGLSS is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with this program.  If not, see <http://www.gnu.org/licenses/>.

if isscalar(bounds),
    min_len = 0;
    max_len = bounds;
else
    min_len = bounds(1);
    max_len = bounds(2);
end

states = n_states(f);

forwards = true;    % Assume generating forward
if nargin < 3,
    curstates = start_state( f );   % List of states reachable after n symbols
    do_chunks = false;
else
    do_chunks = true;
    if isempty(curstates),
        curstates = start_state(f);
    elseif isscalar(curstates) && curstates < 0,
        forwards = false;
        curstates = find(f.links(:, 1) < 0);
    else
        if any(curstates < 1) || any(curstates > states),
            error('FSG:badCurstates', ...
                'Curstates must be from 1 to %d for this grammar', states);
        end
    end
end

symset = symbols(f);
if isempty(symset),
    error( 'Symbols must be specified for FSG to generate grammatical strings' );
end

f = reset_state( f );

curstrings = char('');      % Corresponding list of the symbol sequences
curlen = size(curstrings, 2);   % Length of those symbol sequences (i.e. n)

strs = {};  % Grammatical strings within lengths specified by bounds

if forwards,
    [linkstates,linksymidx,nextstates] = find(f.links(curstates, :));
    while curlen <= max_len && ~isempty(linkstates),
%     fprintf(' grammatical_strings: curlen=%i, number of strings = %i\n', ...
%         curlen, size(curstrings, 1));

        % First take care of chunks and any end-of-string links
        endlinks = (linksymidx == 1);
        if curlen >= min_len,
            if do_chunks,
                strs = [strs; cellstr(curstrings)];
            elseif any(endlinks),
                if isempty(curstrings),
                    strs = {'.'};   % Return '.' to represent null string
                else
                    strs = [strs; cellstr(curstrings(linkstates(endlinks), :))];
                end
            end
        end

        % Process the real links, if any
        if any(~endlinks),
            syms = symset(linksymidx(~endlinks)-1)';  % Get the symbols for each link

            % Now add symbol from each link to corresponding symbol string
            if isempty(curstrings),
                curstrings = syms;
            else
                % NB. Do NOT use STRCAT because it is SLOW!!
                %             t=clock;
                %             curstrings = strcat(curstrings( linkstates(~endlinks), : ), syms);

                curstrings = [curstrings( linkstates(~endlinks), : ), syms];
                %             fprintf(' grammatical_strings: strcat time = %f\n', etime(clock, t));
            end
            curlen = size(curstrings, 2);   % Length of these symbol sequences
            curstates = nextstates(~endlinks);
            [linkstates,linksymidx,nextstates] = find(f.links(curstates, :));
        else
            linkstates = [];
        end
    end

else
    % generate backwards
    prevstates = cell(states, 1);
    linksymidx = cell(states, 1);
    for i = 1:states,
        [prevstates{i}, linksymidx{i}] = find(f.links == i);
    end
    
    curstrings = char('');
    curlen = 0;
    while curlen < max_len && ~isempty(curstates)
        c = []; % new list of current states
        s = zeros(0, curlen+1); % new character array of generated strings
        for i = 1:states,
            tgt_idx = find(curstates == i);
            if ~isempty(tgt_idx)
                [X, prev] = meshgrid(tgt_idx, prevstates{i});
                c = [c; prev(:)];

                [X, symidx] = meshgrid(tgt_idx, linksymidx{i});
                syms = symset(symidx(:)-1)';  % Get the symbols for each link
                if isempty(curstrings),
                    newstrings = syms;
                else
                    newstrings = [syms, curstrings(X, :)];
                end
                s = [s; newstrings];
            end
        end
        curlen = size(s, 2);   % Length of these symbol sequences
        [curstrings, idx] = unique(s, 'rows');
        curstates = c(idx);
        if curlen >= min_len,
            strs = [strs; cellstr(curstrings)];
        end
    end
end

gstrings = char(unique(strs));
f.strings = gstrings;
f.string_bounds = [min_len, max_len];
return

end
