function [s, f] = link( f, state1, symbol, s );
% LINK  Get/set links from one state to another for given symbol(s)
%
% tgt_states = link( fsg ) returns a matrix of target states reachable from
%   the states of FSG.  There will be one row per state of FSG, one
%   column for the end-of-string symbol '' plus one column for each of the
%   proper symbols of FSG, in order.
%
% tgt_states = link(fsg, src_states) returns a matrix of target states
%   reachable from the specified vector of src_states.  There will be one
%   row per state of FSG, one column for the end-of-string symbol '' plus
%   one column for each of the proper symbols of FSG, in order.
%
% tgt_states = link(fsg, src_states, symbols) returns a matrix of target states
%   reachable from the specified src_states via the specified symbols.
%   Symbols can be specified either as a string, or as a vector of symbol
%   indices, where 1 represents the end-of-string symbol.  The
%   matrix returned has one row for each src_state and one column for each
%   symbol.
%
% tgt_states = link(fsg, [], symbols) returns a matrix of target states
%   reachable from any state of FSG by the specified symbols.
%
% newf = link(fsg, src_states, symbols, tgt_states) 
% [s, newf] = link(fsg, src_states, symbols, tgt_states) 
%   sets links from the specified vector of src_states via the
%   corresponding vector of symbols to the target states specified in the
%   tgt_states matrix (one row per state, one column per symbol).  If there
%   are two output arguments, the modified FSG is returned as the second
%   output argument.  SYMBOLS can be '' or can include '.' to specify an
%   end-of-string transition, which should usually terminate in target
%   state -1.

% Copyright 2007 Todd M. Bailey
%
% This file is part of AGL StimSelect (AGLSS).
%
%     AGLSS is free software; you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation; either version 3 of the License, or
%     (at your option) any later version.
% 
%     AGLSS is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with this program.  If not, see <http://www.gnu.org/licenses/>.

if nargin < 2,
    s = f.links;
else
    if isempty(state1),
        state1 = 1:n_states(f);
    end
    if nargin < 3,
        s = f.links(state1, :);
    else
        s_index = symbol_index( f, symbol );
        if nargin < 4,
            s = f.links(state1, s_index);
        else
            f = add_links(f, state1, s_index, s);
            if nargout < 2,
                s = f;
            end
        end
    end
end
return

end


function f = add_links(f, state1, s_index, s)
% Warn if changing a link
if any(f.links(state1, s_index) ~= 0),
    warning('FSG:nondeterministic', ...
        ['Changing symbol''s destination state ', ...
        'might be attempt to define nondeterministic grammar']);
end

% set the new link value(s)
f.links(state1, s_index) = s;

if all(s_index < 2),
    % no real links, just end-of-string
    return
end

% Make sure n_states includes biggest state referenced
s_max = max(s(:));
if s_max > n_states(f),
    warning('FSG:implicit_newstate', ...
        'Implicitly adding new state');
    f.links(end+1:s_max, :) = 0;    % allocate space for new states
end

% add upstream info if possible, else void it
% % s = f.links(state1, s_index);   % expand s to matrix if necessary

for i = 1:length(state1),
    cur_start = state1(i);
    for j = 1:length(s_index),
%         cur_sym = s_index(j);
        cur_dest = f.links(i, j);
        if cur_dest > 0 && ~any(f.links(cur_dest,2:end)),
            % Adding a link to deadend (new?) state -- add
            % corresponding upstream states too
            [nil, f] = upstream_states(f, cur_dest, ...
                [upstream_states(f, cur_start), cur_start], 'add');
        else
            % Deleting a link, or adding link to state that already
            % has links -- upstream data will be invalid
            [nil, f] = upstream_states(f, [], [], 'reset');
            return
        end
    end
end

end
