function [u, f] = upstream_states(f, s, u, option)
% UPSTREAM_STATES   Get/set stored list of states upstream of given state
%
% C = UPSTREAM_STATES(F) returns a cell array, each element i of which
% contains a vector listing the states of FSG F that are upstream of state i.
%
% V = UPSTREAM_STATES(F, S) returns a vector listing the states in FSG F
% that are upstream of state S.
%
% [V, F] = UPSTREAM_STATES(F, S, V) sets the list of upstream states for
% state S to vector V.
%
% [V, F] = UPSTREAM_STATES(F, S, V, 'add') adds the states indicated in 
% vector V to the list of upstream states for state S (using set union).
%
% [C, F] = UPSTREAM_STATES(F, [], C) sets upstream states for all states of
% F according to the vectors in the corresponding cells of cell array C.
%
% [nil, F] = UPSTREAM_STATES(F, [], [], 'reset') invalidates the stored list,
% which will be recomputed from scratch when next needed.  The first output
% argument will be {}.

% Copyright 2007 Todd M. Bailey
%
% This file is part of AGL StimSelect (AGLSS).
%
%     AGLSS is free software; you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation; either version 3 of the License, or
%     (at your option) any later version.
% 
%     AGLSS is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with this program.  If not, see <http://www.gnu.org/licenses/>.

if isempty(f.upstream),
    if nargin < 4 || ~strcmp(option, 'reset'),
        f.upstream = paths( link(f) );
    end
end

if nargin < 2,
    u = f.upstream;
else
    if s > length(f.upstream),
        % Initialize cells if necessary
        f.upstream(end+1:s) = {repmat([],s-length(f.upstream),1)};
    end
    switch nargin,
        case 2,
            u = f.upstream{s};
        case 3,
            u = unique(u);  % guarantees ascending order, too
            if s,
                f.upstream{s} = u;
            else
                f.upstream = u;
            end
        otherwise
            switch option,
                case 'add',
                    u = unique(u);  % guarantees ascending order, too
                    f.upstream{s} = union_sorted(f.upstream{s}, u);
                case 'reset',
                    f.upstream = {};
                    u = f.upstream;
                otherwise,
                    error('Invalid option');
            end
    end
end
end



function u = paths( links )
% Compute upstream states directly from the full set of transitions
n = size(links, 1);   % n_states
links = links(:, 2:end);    % Ignore end-of-string transitions

[src_states, nil, tgt_states] = find(links);
connect = sparse(src_states, tgt_states, 1, n, n);

paths = connect;
newpaths = connect;
while any(newpaths(:)),
    [src, tgt] = find(newpaths);
    newpaths = zeros(n);
    for i = 1:length(tgt),
        s = src(i);
        t = tgt(i);
        [nil, tgt2, pathlen] = find(paths(t,:));
        newtgt = tgt2(paths(s, tgt2) == 0);
        newpathlen = pathlen(paths(s, tgt2) == 0);
        if newtgt,
            newpaths(s, newtgt(newtgt~=s)) = 1;   % Do not follow loops back farther
            paths(s, newtgt) = newpathlen + 1;
        end
    end
end

for s = 1:n,
    u{s} = reshape(find(paths(:,s)), 1, []);
end
end