function [n_bits, e] = encode(n)
% ENCODE    Encode natural numbers efficiently as binary strings
%   N_BITS = ENCODE(N), for a single nonnegative integer N, returns
%   the length of a string, E(1:i), of '0's and '1's that encode N.  The
%   values 0, 1, 2, 3, 4, ... are encoded as '', '0', '1', '00', '01', ... .  
%   This is slightly more efficient than the simple binary representation of 
%   N because leading zeros are not ignored.  This saves one bit on most numbers.
%
%   [N_BITS, E] = ENCODE(N) returns the string itself as well as its
%   length.  This takes more time than simply computing the length.
%   
%   This implements the correspondence between numbers and binary strings 
%   described by Li & Vitanyi (1997, p. 12, Eq. 1.1).
%   In particular, N = 2^i - 1 + E(1)*2^0 + E(2)*2^1 + ... E(i+1)*2^i.
%
%   Use N = DECODE(E) to recover N from its encoding.

% Copyright 2007 Todd M. Bailey
%
% This file is part of AGL StimSelect (AGLSS).
%
%     AGLSS is free software; you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation; either version 3 of the License, or
%     (at your option) any later version.
% 
%     AGLSS is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with this program.  If not, see <http://www.gnu.org/licenses/>.

if floor(n) ~= n,
    warning('Encode called with non-integer');
    n = round(n);
end
if n < 0,
    error('Encode called with negative value');
end

n_bits = ceil(log2(n+2))-1;
if nargout > 1,
    n_excess = n - 2^(n_bits) + 1;
    e = dec2bin(n_excess, n_bits);
end