function [n_bits, e] = prefix_code(n, iters)
% PREFIX_CODE   Encode natural numbers as self-delimiting bit strings
%
% N_BITS = PREFIX_CODE(N), for non-negative integer N, returns the length
%   of a self-delimiting binary code representing N as a string of L 1's
%   followed by a delimiting 0, followed by L bits encoding N as in ENCODE(N).  
%   For example, N=6 => '11011', requires 5 bits.
%
% N_BITS = PREFIX_CODE(N, ITERATIONS) returns the length of a recursively 
%   iterated prefix code representing N.  The first iteration represents N
%   by a string of L 1's, a delimiting 0, and L bits representing N itself.
%   Thus, the string of L 1's indicates (by its length) the number of
%   relevant bits following the delimiting 0.  Each higher-order iteration
%   begins with a string of 1's followed by a delimiting 0, where the
%   number of 1's indicates the number of following bits, which are a
%   binary representation of L(i-1), which indicates the number of
%   following bits representing L(i-2), and so on, until the last set of
%   bits is a binary encoding of N itself.  This is the encoding scheme
%   described in Li & Vitanyi (1997, Eq. 1.2, p. 13).
%
%   If ITERATIONS is 0, the code is simply N 1's followed by a delimiting
%   0.
%
% [N_BITS, E] = PREFIX_CODE(...) returns the string itself (as a character
%   array), as well as its length.  This takes more time than simply
%   computing the length of the encoding.

% Copyright 2007 Todd M. Bailey
%
% This file is part of AGL StimSelect (AGLSS).
%
%     AGLSS is free software; you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation; either version 3 of the License, or
%     (at your option) any later version.
% 
%     AGLSS is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with this program.  If not, see <http://www.gnu.org/licenses/>.

if nargin < 2,
    iters = 1;
end

if iters > 0,
    if nargout < 2,
        n_suffix = encode(n);
        n_prefix = prefix_code(n_suffix, iters-1);
        n_bits = n_prefix + n_suffix;
    else
        [n_suffix, e_suffix] = encode(n);
        [n_prefix, e_prefix] = prefix_code(n_suffix, iters-1);
        n_bits = n_prefix + n_suffix;
        e = [e_prefix, e_suffix];
    end
else
    n_bits = n+1;   % N 1's plus a delimiting 0
    if nargout > 1,
        e = [repmat('1', 1, n), '0'];
    end
end