function d = EditDist(s1,s2,varargin)
%EDITDIST Finds the Edit Distance between strings s1 and s2. The Edit Distance
%         is defined as the minimum number of single-character edit operations 
%         (deletions, insertions, and/or replacements) that would convert 
%         s1 into s2 or vice-versa. Uses an efficient dynamic programming
%         algorithm. Useful for gene sequence matching, among other applications.
%
%         Example: d = EditDist('cow','house') returns a value of 4.
%         Example: s1 = 'now'; s2 = 'cow'; EditDist(s1,s2) returns a value of 1.
%         Example from gene sequence matching:
%         EditDist('ATTTGCATTA','ATTGCTT') returns a value of 3.
%         
%         If there are more than two inputs, the 3d, 4th, and 5th inputs will be
%         interpreted as the costs of the three edit operations: DELETION,
%         INSERTION, and REPLACEMENT respectively. The default is 1 for all
%         three operations. Note that if the cost of replacement is at least twice
%         the respective costs of deletion and insertion, replacements will never be 
%         performed.
%
%         Example: EditDist('cow','house',1,1,1) returns a value of 4.
%         Example: EditDist('cow','house',1,2,1.5) returns a value of 5.
%         Example: EditDist('cow','house',1,1,2) returns a value of 6.
%
%If either s1 or s2 (or both) are cell arrays of strings, EditDist returns
%a matrix of pairwise distances between the items in s1 and s2.  The
%distance matrix will have a row for each item in s1 and a column for each
%item in s2.
%
%
%USAGE:   d = EditDist('string1','string2');
%
%         d = EditDist('string1,'string2',1.5,1,2);
%
%

%Written and tested in Matlab 5.3, Release 11.1 (should work with earlier versions).
%Copyright 2000, Miguel A. Castro 6/4/2000
%talk2miguel@yahoo.com
%
% Copyright 2007 Todd M. Bailey
%
% This file is part of AGL StimSelect (AGLSS).
%
%     AGLSS is free software; you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation; either version 3 of the License, or
%     (at your option) any later version.
% 
%     AGLSS is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with this program.  If not, see <http://www.gnu.org/licenses/>.

% 6 Sept 2007, Todd M. Bailey:
%   Swapped DEL and INS to get directionality going from s1 to s2
%
%24 Aug 2006, Todd M. Bailey:
%Added code to allow s1 or s2 to be cell array of strings.
%
%Attempted slightly improved performance in Matlab 7.2 (should work with earlier
%versions), Todd M. Bailey 24 Aug 2006, BaileyTM1@Cardiff.ac.uk
%Loops to initialize first row and column of matrix D were replaced with
%vector operations.  However, these give slower performance for shortish
%strings, so reverted to original code.
%Also replaced & with && operator to make lint happier.
%------------------------------------------------------------------------------------------

%Deal with cell arrays of strings if necessary.
%Uses one level of recursion.
if iscell(s1) || iscell(s2),
    if ~iscell(s1),
        s1 = {s1};
    elseif ~iscell(s2),
        s2 = {s2};
    end
    n1 = length(s1);
    n2 = length(s2);
    d = zeros(n1,n2);
    for i=1:n1,
        for j=1:n2,
            d(i,j) = EditDist(s1{i}, s2{j}, varargin{:});
        end
    end
    return
end


%Determine the number of inputs. If 2 inputs, set default edit costs to 1.
%Otherwise, make sure there are exactly 5 inputs, and set edit costs accordingly.
if ~isempty(varargin)
   if length(varargin) ~= 3
      error('Usage is: EditDist(''string1'',''string2'',DeleteCost,InsertCost,ReplaceCost)');
   end;
   DelCost = varargin{1};
   InsCost = varargin{2};
   ReplCost = varargin{3};
else
   DelCost = 1;
   InsCost = 1;
   ReplCost = 1;
end;

[m1,n1] = size(s1);
[m2,n2] = size(s2);

%Make sure input strings are horizontal.
if ~(ischar(s1) && ischar(s2) && m1 == 1 && m2 == 1)  
   error('s1 and s2 must be horizontal strings.');
end;

%Initialize dynamic matrix D with appropriate size:
D = zeros(n1+1,n2+1);

%This is dynamic programming algorithm:
for i = 1:n1
   D(i+1,1) = D(i,1) + DelCost;
end;
% D(:, 1) = DelCost * (0:n1);   % faster for n1=100, but not n1=10

for j = 1:n2
   D(1,j+1) = D(1,j) + InsCost;
end;
% D(1, :) = InsCost * (0:n1);   % faster for n2=100, but not n2=10

for i = 1:n1
   for j = 1:n2
      if s1(i) == s2(j)
         Repl = D(i,j);
      else
         Repl = D(i,j)+ReplCost;
      end;
%       D(i+1,j+1) = min([D(i,j)+Repl D(i+1,j)+DelCost D(i,j+1)+InsCost]);
      % Avoid min() for sake of speed
      Del = D(i,j+1)+DelCost;
      Ins = D(i+1,j)+InsCost;
      if Repl <= Del,
          if Repl <= Ins,
              Cost = Repl;
          else
              Cost = Ins;
          end
      elseif Del <= Ins,
          Cost = Del;
      else
          Cost = Ins;
      end
      D(i+1,j+1) = Cost;
   end;
end;

d = D(n1+1,n2+1);