function [G, CX] = grammatical_chunks(f, chsize, type)
% GRAMMATICAL_CHUNKS   Identify grammatical chunks for a FSG
%
% G = grammatical_chunks(fsg, chsize)
% G = grammatical_chunks(fsg, chsize, 'chunk_type')
% [G, CX] = grammatical_chunks(...)
%
%   Returns a logical matrix indicating which possible chunks composed of
%   the symbols of a finite state grammar are grammatical.
%
%   FSG should be an FSG object.
%   
%   CHSIZE specifies the size of chunks of interest.
%
%   'chunk_type' should be one of 'all', 'anchor', 'initial', or 'final',
%   to indicate grammaticality of all chunks, anchor chunks (those
%   occurring at the beginning or end of grammatical strings), initial
%   chunks only, or final chunks only.  The default is 'all'.
%
% [G, CX] = grammatical_chunks(...)
%   Also returns a character array listing the possible chunks.
%   G(j) is true if chunk CX(j,:) is grammatical.
%
%   SEE ALSO: grammatical_strings(), chunk_names().

% Copyright 2007 Todd M. Bailey
%
% This file is part of AGL StimSelect (AGLSS).
%
%     AGLSS is free software; you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation; either version 3 of the License, or
%     (at your option) any later version.
% 
%     AGLSS is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with this program.  If not, see <http://www.gnu.org/licenses/>.

if ~isscalar(chsize),
    error('Single value expected for chunk size');
end
minmaxsize = [chsize chsize];

if nargin < 3 || isempty(type),
    type = 'all';
else
    type = lower(type);
end

syms = symbols(f);

CX = chunk_names(syms, chsize);
CX = CX{chsize};

switch type,
    case 'all',
        g_chunks = grammatical_strings(f, minmaxsize, 1:n_states(f));
    case 'initial',
        g_chunks = grammatical_strings(f, minmaxsize, []);
    case 'final',
        g_chunks = grammatical_strings(f, minmaxsize, -1);
    case 'anchor',
        g_chunks = union( ...
            grammatical_strings(f, minmaxsize, []), ...
            grammatical_strings(f, minmaxsize, -1), 'rows' );
    otherwise
        error('Illegal chunk type');
end
G = ismember(CX, g_chunks, 'rows');
if chsize > 1,
    G = reshape(G, repmat(length(syms), 1, chsize));
end

end
