function [totalchi2, indivchi2] = likelihoodchi2(obsvals, expvals)
% LIKELIHOODCHI2   Compute log likelihood chi-square for contingency table
%
% [TOTALCHI2, INDIVCHI2] = LIKELIHOODCHI2(CONTINGENCIES)
% [TOTALCHI2, INDIVCHI2] = LIKELIHOODCHI2(OBSERVED, EXPECTED)
%
% [TOTALCHI2, INDIVCHI2] = LIKELIHOODCHI2(CONTINGENCIES)
%   Computes a log likelihood chi-square statistic for a 2D contingency table,
%   returning the overall chi-square statistic (TOTALCHI2), as well as
%   individual chi-square components for each cell of the table.  
%
%   The individual components are 2*obs*log(obs/exp), where the observed
%   frequencies are the values in the contingency table, and expected
%   frequencies are computed by multiplying row and column totals and
%   dividing by the overall total for the whole table (cf. Howell, 1992, p.
%   134).
%
% [TOTALCHI2, INDIVCHI2] = LIKELIHOODCHI2(OBSERVED, EXPECTED)
% [TOTALCHI2, INDIVCHI2] = LIKELIHOODCHI2(OBSERVED, [])
%   Computes a log likelihood chi-square statistic for a set of data
%   relative to a given set of expected values.  The expected values array
%   must be the same size as the array of observed values, except that if
%   the array of expected values is [], the observed values are compared
%   against the overall grand mean value (i.e. the same value is expected
%   for every observed value).
%
% Log likelihood chi-square statistics are less sensitive to small samples 
% than the usual Pearson chi-square statistic is (Howell, 1992, p. 144).

% Copyright 2007 Todd M. Bailey
%
% This file is part of AGL StimSelect (AGLSS).
%
%     AGLSS is free software; you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation; either version 3 of the License, or
%     (at your option) any later version.
% 
%     AGLSS is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with this program.  If not, see <http://www.gnu.org/licenses/>.

% nrows and ncols are no longer used, because 0*obsvals is faster than
% zeros(nrows,ncols)
% [nrows, ncols] = size(obsvals);

ctab = (nargin < 2);    % Interpret obsvals as contingency table?

%This check has been removed for speed
% if any(obsvals(:) < 0),
%     error('Negative values not allowed');
% end

if ctab,
    % Compute expected values
    ttotal = sum(obsvals(:));
    trows = sum(obsvals,2);
    tcols = sum(obsvals,1);
    if ttotal == 0,
        % special handling for matrix of all zeros
        expvals = 0 .* obsvals;
    else
        expvals = trows * tcols ./ ttotal;
    end
elseif isempty(expvals),
    % Compare against grand mean
%     expvals = ones(size(obsvals)) * mean(obsvals(:));
    % avoid call to ones and size for speed:
    expvals = ones(size(obsvals)) .* mean(obsvals(:));
end

% Compute log likelihood chi-square stats to determine who is
% under-represented in which positions.  Note that 0*log(0) is 0
nz = find(obsvals);    % Nonzeros require the usual computation
obs_nz = obsvals(nz);
indivchi2 = 0 .* obsvals;
indivchi2(nz) = 2 .* obs_nz .* reallog(obs_nz ./ expvals(nz));
% z = find(obsvals == 0);
% indivchi2(z) = 0;

totalchi2 = sum(indivchi2(:));
% indivchi2 = reshape(indivchi2, nrows, ncols);
