function freq = ngram(items, syms, sizes, freq)
% NGRAM  Count frequency of ngrams of particular lengths
%
% F = ngram(I, S, L)
%
% F = ngram(I, S, L, F0)
%
% Counts the frequency of sequences of symbols from vector S in items I.
% Vector L specifies the sequence lengths to be counted.  
%
% If L is a scalar, then ngram(I, S, L) returns a matrix of dimension
% length(S)^L containing frequency counts F(d1,d2,...,dL) corresponding to
% sequences S(d1)S(d2)...S(dL) appearing as subsequences in item(s) I.
%
% If L is a vector of length greater than 1, then ngram(I, S, L) returns a
% cell array of frequency matrices, one for each sequence length specified in L.
%
% Items I can be a cell array of strings, a CHAR matrix of
% strings (or a single string), or a matrix of indices into S
% (right-padded with zeros).
%
% If F0 is given, it should have the same form as the expected return value
% F (a frequency matrix or cell array of such matrices).  The values
% specified are used as initial frequency counts.  If F0 is not given or is
% given as [], counting begins from 0.
%
%
% Therefore NGRAM is creating an output matrix which contains how often
% a ngram (A to B) appears inside ITEMS, giving back a matrix or matrices
% of DESIRED_SYMS.
%
% An n-gram is a sub-sequence of n items from a given sequence. An n-gram 
% of size 1 is a "unigram"; size 2 is a "bigram" (or, more appropriately 
% but less commonly, a digram); size 3 is a "trigram"; and size 4 or more 
% is simply called an "n-gram" or "(n ? 1)-order Markov model".
% For sequences of characters, the 3-grams (sometimes spelled "trigrams")
% that can be generated from "good morning" are "goo", "ood", "od ", 
% "d m", " mo", "mor" and so forth.
%
% NOTE: the input string DESIRED_SYMS must contain all symbols used inside
% ITEMS
%
% Example:
%
% SET1=['ABA ';'AABB';'BAB '];
% grams=ngram(SET1,['AB '],2) % with a blank at the end! 
%
%   grams = 
%
%        'Not calculated!'
%             [2x2 double]
%
% grams{2}
%
%   ans =
%
%         1     3
%         2     1

% Copyright 2007 Todd M. Bailey
%
% This file is part of AGL StimSelect (AGLSS).
%
%     AGLSS is free software; you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation; either version 3 of the License, or
%     (at your option) any later version.
% 
%     AGLSS is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with this program.  If not, see <http://www.gnu.org/licenses/>.

if nargin < 4,
    freq = [];
end

% transformation to integer numbers. Is rather slow.  Could be left out if 
% input is already in doubles / integers
items = double(char(items));

% get the number of rows and columns
[n_items, item_length] = size(items);
n_syms = length(syms);

% the letters are in the form: A -> 65 , B -> 66 ...
% Now we are converting them to number 1 , 2 .. number of the last
% letter (in general a blank)
% in addition the matrix of the letters is converted into a single
% vector (see the line with reshape)
syms(end+1) = ' ';
sym_idx(syms)=[1:n_syms 0];

strs = sym_idx(items);
% strs = sym_idx(reshape(items',1,[]));

n_sizes = length(sizes);

if n_sizes == 1,
    if sizes <= 3,
        freq = gram_13(strs, n_syms, n_items, item_length, ...
            sizes, freq);
    else
        freq = gram_n(items, syms, n_items, item_length, sizes, freq);
    end
else
    % preallocation of output variable
    if isempty(freq),
        freq = cell(n_sizes,1);
    end
    for size_idx = 1:n_sizes,
        gram_size = sizes(size_idx);
        if gram_size <= 3,
            freq{size_idx} = gram_13(strs, n_syms, n_items, item_length, ...
                gram_size, freq{size_idx});
        else
            freq{size_idx} = gram_n(items, syms, n_items, item_length, ...
                gram_size, freq{size_idx});
        end
    end
end

%% -----------------------------------------------------------

function freq = gram_13(strs,n_syms,n_items,item_length,chunk_size,freq)

% preallocation of the output variable
if isempty(freq),
    switch chunk_size,
        case 1,
            freq = zeros(1, n_syms);
        case 2,
            freq = zeros(n_syms);
        otherwise
            freq = zeros(n_syms, n_syms, n_syms);
    end
end

% len_strs = length(strs);

for item = 1:n_items,
    for chunk_end = item_length : -1 : chunk_size,
        ch_end = strs(item, chunk_end);
        if ch_end,
            switch chunk_size
                case 1,
                    freq(ch_end) = freq(ch_end) + 1;
                case 2,
                    ch1 = strs(item, chunk_end - 1);
                    freq(ch1, ch_end) = freq(ch1, ch_end) + 1;
                otherwise,
                    ch1 = strs(item, chunk_end - 2);
                    ch2 = strs(item, chunk_end - 1);
                    freq(ch1, ch2, ch_end) = freq(ch1, ch2, ch_end) + 1;
            end
        end
    end
end

%% -----------------------------------------------------------

function out= gram_n (items,syms,m,n,gram_size,out)

items=double(char(items));

len_syms=length(syms);

one_mat = full(sparse(m,gram_size-1)) + 32;
items = [items one_mat];
sym_idx(syms)=1:length(syms);
items = reshape(items',1,[]);
items=sym_idx(items);

if nargin < 6 || isempty(out),
    out=zeros((len_syms)*ones(1,gram_size));
end

c_size = size(out);
dim_offsets = cumprod( [1, c_size(1 : end-1)] );

for count = 1: (length(items)- gram_size + 1)
    for i=2:length(dim_offsets)
        idx(i) = (items(count + i - 1) - 1);
    end
    ind=idx(2:end)* dim_offsets(2:end)';
    ind = ind + items(count);
    out(ind) = out(ind) + 1;
end

% -----------------------------------------------------------
