function [tf,loc] = ismember_sorted(a,s)
%ISMEMBER_SORTED   True for member of sorted set.
% ISMEMBER_SORTED(A,S) for the cell array of strings, A, returns a matrix
% the same size as A containing 1 where the elements of A are in the set S
% and 0 otherwise.
% S must be a sorted cell array of strings.
%
% [TF,LOC] = ISMEMBER_SORTED(A,S) also returns an index array LOC where
% LOC(i) is the index in S which matches A(i) (highest if there are ties)
% or, if there is no such index, -LOC(i) is the smallest index in S which
% is greater than S(i).
%
% See also ISMEMBER, MATCH_SORTED, INTERSECT_SORTED, SETDIFF_SORTED, UNION_SORTED.

% Copyright 2007 Todd M. Bailey
% Written by Todd M. Bailey, based on Tom Minka's function for numeric
% sets (in the Lightspeed Toolbox).
%
% This file is part of AGL StimSelect (AGLSS), but is covered by a
% Microsoft Research Shared Source license agreement (MSR-SSLA).  You
% should have received a copy of that agreement along with this file.  If
% not, see
% http://research.microsoft.com/research/downloads/Details/92f6d4ed-4df4-43
% 69-9b92-89b28da8795d/License.txt .
% 
% This file is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% MSR-SSLA for more details.
%
% You may use, copy, reproduce, and distribute this Software for any
% non-commercial purpose, subject to the restrictions in the MSR-SSLA. 
%
%
% This file is derived from a file of the same name in the LightSpeed
% Toolbox, listed as "Fast subroutines for Matlab programs" at
% http://research.microsoft.com/research/downloads .
%
% This file has been changed from its original form, on 4 September 2007, 
% to work on cell arrays, primarily by the addition of the two
% sub-functions below.


if nargout < 2
  tf = ismembc_sorted(a,s);
else
  loc = ismembc2_sorted(a,s);
  tf = (loc > 0);
end

end


function tf = ismembc_sorted(a,s)
% assumes a and s are both sorted cell arrays of strings
tf = false(size(a));
numelA = numel(a);
numelS = numel(s);

left = 1;
for i = 1:numelA,
    right = numelS;
    while left <= right,
        mid = floor((left+right)/2);
        leg = strleg(a{i}, s{mid});
        if leg > 0,
            left = mid+1;
        elseif leg < 0,
            right = mid-1;
        else
            tf(i) = true;
            break
        end
    end
end
end


function loc = ismembc2_sorted(a,s)
% assumes a and s are both sorted cell arrays of strings
loc = zeros(size(a));
numelA = numel(a);
numelS = numel(s);

left = 1;
for i = 1:numelA,
    right = numelS;
    while left <= right,
        mid = floor((left+right)/2);
        leg = strleg(a{i}, s{mid});
        if leg > 0,
            left = mid+1;
        elseif leg < 0,
            right = mid-1;
        else
            loc(i) = mid;
            break
        end
    end
    if loc(i) == 0,
        loc(i) = -left;
    end
end
end
