function s = aglss( arg1, varargin )
% AGLSS - Make an AGL Stim Select object
%
% AGLSS( grammar, ... )
% AGLSS( grammar, ..., max_items, ... )
% AGLSS( grammar, ..., [min_len max_len], ... )
%   Returns an object of class AGLSS whose grammatical items will be based
%   on GRAMMAR, which should be an object of class FSG.  
%
%   MAX_ITEMS, if given, specifies the maximum number of items to consider.
%   May be Inf, if MIN_LEN and MAX_LEN are specified. The default value is
%   10000.
%
%   MIN_LEN and MAX_LEN, if given, specify the minimum and maximum string
%   lengths to consider.  If not specified, MIN_LEN defaults to the
%   MAXCHSIZE property of the AGLSS object (MAXCHSIZE defaults to 3).
%   MAX_LEN defaults to the smallest value (at least as large as MIN_LEN)
%   such that the total number of grammatical and ungrammatical strings of
%   length MIN_LEN to MAX_LEN is at least as great as MAX_ITEMS.  If
%   MAX_ITEMS is Inf, MIN_LEN and MAX_LEN must be specified.
%
%   Example: AGLSS(pothos_bailey_grammar(), 10000, [4 7]);
%
% AGLSS( names, isG )
%   Returns an object of class AGLSS whose grammatical items are NAMES(isG)
%   and whose non-grammatical items are NAMES(~isG). NAMES should be a cell
%   array of strings. ISG should be a logical vector of the same length  as
%   NAMES.
%
%   Example: AGLSS(grammatical_strings(xx_grammar(), [3 5]), ...
%                isgrammatical(xy_grammar, ...
%                   grammatical_strings(xx_grammar(), [3 5])))
%
% AGLSS( AGLSS )
%   Given an object of class AGLSS, AGLSS just returns the object.
%
% AGLSS()
%   Returns an empty object of class AGLSS.

% Copyright 2007, 2013 Todd M. Bailey
%
% This file is part of AGL StimSelect (AGLSS).
%
%     AGLSS is free software; you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation; either version 3 of the License, or
%     (at your option) any later version.
% 
%     AGLSS is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with this program.  If not, see <http://www.gnu.org/licenses/>.

if nargin && isa(arg1, 'aglss'),
    s = arg1;
    return
end

% declare fields for the object, with tentative initial values
% % s.grammar = [];     % wants to be an FSG object
s.grammarname = '';
s.max_items = 10000;    % max number of potential items (G + NG)
s.minmaxlen = [];   % [min_len] or [min_len max_len] of potential items

s.itemnames = {};       % placeholder for item names if specified directly
% % s.s_isG = logical([]);  % placeholder for indicators of G item names

s.items = [];   % placeholder for potential item structures (see ITEMSET_INIT)
s.isG = logical([]);    % placeholder for indicators of G items

s.ngram_max = 3;    % Max chunk size, default to trigrams
s.chunks = {};      % placeholder for possible chunks of various sizes
s.chfreq_g = {};    % placeholder for chunk frequencies of G items

s.tgt_fcats = {};   % placeholder for frequency categories per chunk size
s.n_novel = [];     % placeholder for minimum novel chunks of each size
s.novel_pmax = [];  % max fraction of novel chunks (of each size) per item
s.fwt = {};         % placeholder for frequency-weight targets

s.train = [];       % placeholder for training item structure
s.test = [];        % placeholder for test item structure


grammar = fsg();    % default

f = factorset();

% inherit FSG and FACTORSET objects
s = class(s, 'aglss', grammar, f);

% initialize the object properly
s = items_init(s, 'reset');


if nargin,
    if isa(arg1, 'fsg'),
% %         s.grammar = arg1;
        grammar = arg1;
        s.grammarname = inputname(1);
    
        % interpret args, if any
        for i = 1:length(varargin),
            arg = varargin{i};
            if isnumeric(arg) && numel(arg) <= 2,
                if isscalar(arg),
                    % max_items
                    s.max_items = arg;
                else
                    % min_len and max_len
                    s.minmaxlen = arg;
                end
            else
                error('StimSelect:AGLSS:Usage', ...
                    'Additional arguments must be [MIN_LEN MAX_LEN] or MAX_ITEMS');
            end
        end
        s = item_property(s, 'grammar', grammar);
    elseif iscellstr(arg1) || ischar(arg1),
        % list of item names to use, accompanied by vector indicating which
        % are grammatical
        
        if length(varargin) < 1,
            error('StimSelect:AGLSS:Usage', ...
                'Itemnames must be accompanied by logical isG vector');
        else
            isG = varargin{1};
        end
        s = item_property(s, 'itemnames', arg1, 'isg', isG);
    else
        error('StimSelect:AGLSS:NotFSG', ...
            'Grammar must be an FSG object or cell array of strings');
    end
end


% s = items_init(s);

end
