function s = choose_items(s, n_train, n_test, headstart, testfam, initialize)
% AGLSS/choose_items - Choose AGL stimulus items
%
% choose_items(aglss, N_T, N_S, N_T0, TF_familiar, TF_initialize)
%  Chooses training items and/or test items for an AGLSS object.  
%
%  N_T specifies the number of training items to be chosen.  
%
%  N_S specifies the number of test items to be chosen for each test set,
%  that is, for each combination of active factors (see
%  AGLSS/FACTORIAL_TESTSETS).  The default is 0.
%
%  N_T0 specifies the number of head start training items to be chosen
%  before selection of test items begins.  The default is 3.
%
%  TF_familiar specifies whether training items are eligible to also be
%  test items.  This might be used in conjunction with FAM_FACTOR.  The
%  default is false, ensuring that all test items are novel.
%
%  TF_initialize specifies whether to reset stimulus selection to start at
%  the beginning (if TRUE or unspecified), or whether to continue stimulus
%  selection and add to previously selected items (if FALSE). The
%  convenience function CHOOSE_MORE_ITEMS calls this function with
%  TF_initialize set to FALSE.
%
% This is an overloaded method for the function CHOOSE_ITEMS.

% Copyright 2007 Todd M. Bailey
%
% This file is part of AGL StimSelect (AGLSS).
%
%     AGLSS is free software; you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation; either version 3 of the License, or
%     (at your option) any later version.
% 
%     AGLSS is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with this program.  If not, see <http://www.gnu.org/licenses/>.

if nargin < 6,
    initialize = true;
end

if nargin < 5,
    testfam = false;
end

if nargin < 4 || isempty(headstart),
    headstart = min(3, n_train);
else
    if n_train < headstart,
        error('StimSelect:ChooseItems:LargeHeadStart', ...
            'Headstart larger than total training items');
    end
end

if nargin < 3,
    n_test = 0;
end

if ~initialize,
    if isempty(s.train) || (s.train.n_items == 0 && s.test.n_items == 0),
        warning('StimSelect:ChooseItems:NotInitialized', ...
            'TF_initialize unexpectedly false, with no prior items');
        initialize = true;
    end
end
if initialize,
    s.train = trainset_init(s.items, s.isG, s.fwt);
end

[s.items, s.train, s.test] = choose_items(s.items, s.train, ...
    headstart, s.test, 0, testfam);
[s.items, s.train, s.test] = choose_items(s.items, s.train, ...
    n_train - headstart, s.test, n_test, testfam);
fprintf('\n');

end
