function res = chunk_property(s, varargin)
% AGLSS/CHUNK_PROPERTY - Get or set StimSelect chunk properties
%
% CHUNK_PROPERTY(aglss)
%   Returns a cell array of property name strings
%
% CHUNK_PROPERTY(aglss, 'property1')
%   Returns the value of the named property
%
% CHUNK_PROPERTY(aglss, 'property1', VAL1, ...)
%   Sets the specified properties of potential items to the given values.
%
% Property should be one of:
%   'maxchsize'     Maximum chunk size
%   'chunks'        Read-only possible chunks of various sizes
%   'chfreq'        Read-only frequency of chunks in G items
%   'tgt_fcats'     Target frequency categories by chunk size
%   'tgt_novel'     Number of reserved novel chunks by chunk size
%   'novel_pmax'    Max fraction of novel chunks per item, by chunk size
%   'tgt_chfreq'    Read-only target frequencies for individual chunks

% Copyright 2007 Todd M. Bailey
%
% This file is part of AGL StimSelect (AGLSS).
%
%     AGLSS is free software; you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation; either version 3 of the License, or
%     (at your option) any later version.
% 
%     AGLSS is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with this program.  If not, see <http://www.gnu.org/licenses/>.

nargs = length(varargin);
switch nargs,
    case 0,
        res = { ...
            'maxchsize', ...
            'chunks', ...
            'chfreq', ...
            'tgt_fcats', ...
            'tgt_novel', ...
            'novel_pmax', ...
            'tgt_chfreq' }';
    case 1,
        switch varargin{1},
            case 'maxchsize',
                res = s.ngram_max;
            case 'chunks',
                res = s.chunks;
            case 'chfreq',
                res = s.chfreq_g;
            case 'tgt_fcats',
                res = s.tgt_fcats;
            case 'tgt_novel',
                res = s.n_novel;
            case 'novel_pmax',
                res = s.novel_pmax;
            case 'tgt_chfreq',
                if isempty(s.fwt),
                    res = [];
                else
                    res = {s.fwt.freq};
                end
            otherwise,
                [msg_id, msg] = pname_error(varargin{1});
                error(msg_id, msg);
        end
    otherwise,
        for v = 1:2:nargs-1,
            pname = lower(varargin{v});
            val = varargin{v+1};
            switch pname,
                case 'maxchsize',
                    if ~(isempty(val) || ...
                            (isscalar(val) && val == floor(val) && val > 0)),
                        error('StimSelect:MaxchsizeType', ...
                            'maxchsize must [] or a positive integer');
                    end
                    if val ~= s.ngram_max,
                        s = chunks_init(s, 'reset');
                        s.ngram_max = val;
                    end
                case 'tgt_fcats',
                    if ~(iscell(val) && length(val) == s.ngram_max),
                        error('StimSelect:Tgt_FcatsType', ...
                            'tgt_fcats must be cell array of length maxchsize');
                    end
                    s = chunks_init(s, 'reset_fwt');
                    s.tgt_fcats = val;
                case 'tgt_novel',
                    if ~(isnumeric(val) && length(val) == s.ngram_max),
                        error('StimSelect:Nchunks', ...
                            'Value must be numeric, of length maxchsize');
                    end
                    if length(val) ~= length(s.n_novel) || ...
                            any(val ~= s.n_novel),
                        s = chunks_init(s, 'reset_fwt');
                        s.n_novel = val;
                    end
                case 'novel_pmax',
                    if ~(isnumeric(val) && length(val) == s.ngram_max),
                        error('StimSelect:Nchunks', ...
                            'Value must be numeric, of length maxchsize');
                    end
                    if length(val) ~= length(s.novel_pmax) || ...
                            any(val ~= s.novel_pmax),
                        s = chunks_init(s, 'reset_fwt');
                        s.novel_pmax = val;
                    end
                otherwise,
                    [msg_id, msg] = pname_error(varargin{v});
                    error(msg_id, msg);
            end
        end
        s = chunks_init(s);
        res = s;    % Return updated object as result
end

end

function [msg_id, msg] = pname_error(badname)
msg_id = 'StimSelect:Chunk_Property';
msg = [badname, ' is not a valid chunk property'];
end
