function s = chunks_init(s, reset)
% AGLSS/CHUNKS_INIT - Initialize set of potential items in AGLSS object
%
% CHUNKS_INIT(aglss)
% CHUNKS_INIT(aglss, 'reset')
%
% SEE ALSO: itemset_init

% Copyright 2007 Todd M. Bailey
%
% This file is part of AGL StimSelect (AGLSS).
%
%     AGLSS is free software; you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation; either version 3 of the License, or
%     (at your option) any later version.
% 
%     AGLSS is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with this program.  If not, see <http://www.gnu.org/licenses/>.

if nargin < 2,
    if isempty(s.items),
        error('StimSelect:Chunks_Init:NoItems', ...
            'Empty or uninitialized potential items');
    end

    % Add info about chunk frequencies
    ngram_max = s.ngram_max;
    if ~isempty(ngram_max) && isempty(s.chunks),
        syms = symbols(s);
        if ~isfield(s.items.item, 'chfreq'),
            % add field if necessary
            s.items = item_property(s.items, ...
                'chfreq', @(x) ngram(x.name, syms, 1:ngram_max));
        end

% %         % Compute possible chunks of various sizes, to help interpret frequency
% %         % matrices returned by ngram().
% %         %   chunks{2} will list the possible bigrams, and so on.
% %         s.chunks = cell(ngram_max, 1);   % allocate space
% %         for chunk_size = 1:ngram_max,
% %             s.chunks{chunk_size} = syms(fullfact(...
% %                 repmat(length(syms), chunk_size, 1)));
% %         end
        s.chunks = chunk_names(syms, ngram_max);
        
        % compute frequency of chunks within G items
        s.chfreq_g = ngram(itemnames(s, s.isG), syms, 1:ngram_max);
        for chsize = 1:length(s.chfreq_g),
            n_chunks = numel(s.chfreq_g{chsize});
            n_zero = sum(s.chfreq_g{chsize}(:) == 0);
            if n_zero,
                fprintf(1,'\t%d of %d chunks of length %d appear in no grammatical items\n', ...
                    n_zero, n_chunks, chsize);
            end
        end
    end

    if ~isempty(ngram_max) && isempty(s.fwt),
        % compute frequency-weight targets for each chunk
        tgt_freqs = s.tgt_fcats;
        if ngram_max > length(tgt_freqs),
            if isempty(tgt_freqs),
                tgt_freqs = cell(1, ngram_max);
            end
% %             curlast = tgt_freqs{end};
% %             for i = length(tgt_freqs):ngram_max,
% %                 tgt_freqs{i} = curlast;
% %             end
        else
            tgt_freqs = tgt_freqs(1:ngram_max); % ignore any extras
        end
        
        n_novel = s.n_novel;
        if ngram_max > length(n_novel),
            n_novel(end+1:ngram_max) = 0;
        else
            n_novel = n_novel(1:ngram_max); % ignore any extras
        end

        novel_pmax = s.novel_pmax;
        if ngram_max > length(novel_pmax),
            novel_pmax(end+1:ngram_max) = 0;  % pad with 0s if needed
        else
            novel_pmax = novel_pmax(1:ngram_max);   % lose extras
        end

        s.fwt = fwt(s.chfreq_g, tgt_freqs, n_novel, novel_pmax);
    end
else
    if strcmpi(reset, 'reset'),
        s.chunks = {};
        s.chfreq_g = {};
        s.fwt = {};         % frequency-weight targets
    elseif strcmpi(reset, 'reset_fwt'),
        s.fwt = {};         % frequency-weight targets
    else
        error('StimSelect:Chunks_Init:UnknownArg', ...
            'Unrecognized argument');
    end
end

end
