function res = item_property(s, varargin)
% AGLSS/ITEM_PROPERTY - Get or set StimSelect potential item properties
%
% ITEM_PROPERTY(aglss)
%   Returns a cell array of property name strings
%
% ITEM_PROPERTY(aglss, 'property1')
%   Returns the value of the named property
%
% ITEM_PROPERTY(aglss, 'property1', VAL1, ...)
%   Sets the specified properties of potential items to the given values.
%
% Property should be one of:
%   'grammar'       Specifies FSG object to define G and NG items (but
%                   note that items might be specified directly without
%                   a grammar)
%   'itemnames'     Specifies or returns item names
%   'max_items'     Scalar maximum number of potential items to consider
%   'lenrange'      Vector of minimum and maximum item lengths
%   'isg'           Read-only logical vector indicating which potential
%                   items are G

% Copyright 2007 Todd M. Bailey
%
% This file is part of AGL StimSelect (AGLSS).
%
%     AGLSS is free software; you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation; either version 3 of the License, or
%     (at your option) any later version.
% 
%     AGLSS is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with this program.  If not, see <http://www.gnu.org/licenses/>.

nargs = length(varargin);
switch nargs,
    case 0,
        res = { ...
            'grammar', ...
            'itemnames', ...
            'max_items', ...
            'lenrange', ...
            'isg' }';
    case 1,
        switch varargin{1},
            case 'grammar',
                res = s.fsg;
            case 'itemnames',
                res = s.itemnames;
            case 'max_items',
                res = s.max_items;
            case 'lenrange',
                res = s.minmaxlen;
            case {'isg', 'isG'},
                res = s.isG(:);    
            otherwise,
                [msg_id, msg] = pname_error(varargin{1});
                error(msg_id, msg);
        end
    otherwise,
        s = items_init(s, 'reset');
        for v = 1:2:nargs-1,
            pname = lower(varargin{v});
            val = varargin{v+1};
            switch pname,
                case 'grammar',
                    if isa(val, 'fsg'),
                        s.fsg = val;
                        s.grammarname = inputname(v+2);
                        s.itemnames = {};
                    else
                        error('StimSelect:AGLSS:NotFSG', ...
                            'Grammar must be an FSG object');
                    end
                case 'itemnames',
                    if ischar(val),
                        val = cellstr(val);
                    end
                    if iscellstr(val),
                        s.itemnames = val;
                        slen = cellfun('length',val);
                        s.minmaxlen = [min(slen) max(slen)];
                        % make empty fsg with appropriate symbols
                        s.fsg = fsg([], unique(strcat(val{:})));
                        s.grammarname = '';
                    else
                        error('StimSelect:AGLSS:NotCellStr', ...
                            'Itemnames must be cell array of strings');
                    end
                case 'symbols',
                    error('StimSelect:AGLSS:ReadOnlyProperty', ...
                        'symbols is a read only property and cannot be set');
                case 'max_items',
                    s.max_items = val;
                case 'lenrange',
                    s.minmaxlen = val;
                case {'isg', 'isG'},
                    s.isG = val(:)';    % ensure row vector
                otherwise,
                    [msg_id, msg] = pname_error(varargin{v});
                    error(msg_id, msg);
            end
        end
        s = items_init(s);
        res = s;    % Return updated object as result
end

end

function [msg_id, msg] = pname_error(badname)
msg_id = 'StimSelect:Item_Property';
msg = [badname, ' is not a valid item property'];
end
