function [s, catnames] = cat_factor(s, fname, cats, catnames)
% CAT_FACTOR - Define a StimSelect item-category factor
%
% S = CAT_FACTOR(aglss, 'fname', C)
% S = CAT_FACTOR(aglss, 'fname', C, N)
%   Creates a factor structure for controlling pre-defined categories of
%   AGL test items.
%
%   The first argument should be an AGLSS object.  It is modified and
%   returned.
%
%   FNAME is the name of the factor.
%
%   C is a 2-dimensional logical array, with C(i,j) indicating whether
%   potential item I is a member of category J.  For mutually exclusive
%   categories, no more than one cell per row should be TRUE.
%
%   N is an optional cell array of strings naming the categories.  N(j)
%   gives the name for category J.  If not supplied, default names are
%   assigned, based on FNAME.
%
% [S,N] = CAT_FACTOR(...)
%   Also returns a cell array of strings naming the categories.  This is
%   possibly useful if N is not specified as an input argument, to get a
%   list of the default category names assigned.

% Copyright 2007 Todd M. Bailey
%
% This file is part of AGL StimSelect (AGLSS).
%
%     AGLSS is free software; you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation; either version 3 of the License, or
%     (at your option) any later version.
% 
%     AGLSS is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with this program.  If not, see <http://www.gnu.org/licenses/>.

if ~isa(s, 'aglss'),
    error('StimSelect:Cat_Factor:Type', ...
        'First argument must be AGLSS object');
end

if ~islogical(cats),
    cats = logical(cats);
end

[n_items, n_cats] = size(cats);

if nargin < 4,
    catnames = make_level_names(fname, n_cats);
elseif ~(iscellstr(catnames) && length(catnames) == n_cats),
    error('StimSelect:Cat_Factor:BadCatNames', ...
        'Category names must be cell array of strings to match number of categories');
end

% Make a cell array with names of factor levels in the first row and
% corresponding factor level structures in the second row
levargs(1, :) = catnames;
for i = 1:n_cats,
    levargs{2, i} = factorlevel(cats(:,i));
end

% All that is required is to identify a priori which potential items are
% eligible for each level of this factor
f = factor_def(levargs{:});

s = factor(s, fname, f );

end
