function [s, catnames, tgt_cats] = chnov_factor(s, fname, ...
    tgt_cats, catnames, n_reserved, chsize)
% CHNOV_FACTOR - Define a StimSelect chunk novelty factor
%
% CHNOV_FACTOR(aglss, 'fname')
% CHNOV_FACTOR(aglss, 'fname', CATS)
% CHNOV_FACTOR(aglss, 'fname', CATS, NAMES)
% CHNOV_FACTOR(aglss, 'fname', CATS, NAMES, M, SIZE)
% CHNOV_FACTOR(aglss, 'fname', CATS, NAMES, [N M], SIZE)
%   Returns a factor structure for controlling the chunk novelty of AGL
%   test items.
%
%   The first argument should be an AGLSS object.  It is modified and
%   returned.
%
%   FNAME is the name to give the factor (e.g. 'ChNov').
%
%   CATS is a vector specifying numbers of novel chunks for different
%   levels of the chunk novelty factor.  The default is [0 1],
%   corresponding to two levels containing zero or one novel chunk per test
%   item, respectively.  That is, some sets of test items will have no
%   novel chunks; other sets of test items will have one novel chunk
%   (i.e. a novel bigram, unless SIZE is specified as something other than
%   2).
%
%   NAMES is a cell array of strings naming the chunk novelty categories.
%   NAMES(j) gives the name for category J.  If not supplied, or [],
%   default names are assigned, based on FNAME.
%   
%   M specifies the minimum number of grammatical chunks to exclude from
%   training items, reserving them to be used as novel chunks in test
%   items.  This allows chunk novelty to be combined factorially with
%   grammaticality.  The default value for M is 2.  
%
%   [N M] specifies the minimum number of chunks (whether grammatical or
%   not) to exclude from training items, and then the minimum number of
%   grammatical chunks to exclude.  In particular, M can be set to 0 if
%   grammaticality is not going to be used as a stimulus factor.
%
%   SIZE specifies the chunk size (> 0) to control.  The default is 2
%   (bigrams).
%
% Potential items for each category of chunk novelty are identified 
% when the factor is defined, by designating a priori which chunks will be
% avoided in training items and counting how many of those chunks occur in
% each potential item.  Because relevant items are identified in advance,
% this factor is very efficient during the item selection process.
%
% SEE ALSO fwt, chstr_factor, ngram

% Copyright 2007 Todd M. Bailey
%
% This file is part of AGL StimSelect (AGLSS).
%
%     AGLSS is free software; you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation; either version 3 of the License, or
%     (at your option) any later version.
% 
%     AGLSS is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with this program.  If not, see <http://www.gnu.org/licenses/>.

if nargin < 6 || isempty(chsize),
    chsize = 2;
end

if nargin < 5,
    n_reserved = [];
end
if isempty(n_reserved),
    nov_g = 2;
elseif isscalar(n_reserved),
    nov_g = n_reserved;
else
    nov_g = n_reserved(2);
end
if length(n_reserved) < 2,
    nov_min = nov_g;
else
    nov_min = max([nov_g, n_reserved(1)]);
end

if nargin < 3 || isempty(tgt_cats),
    tgt_cats = [0 1];
end

if ~isempty(chunk_property(s, 'novel_pmax')),
    error('StimSelect:chnov_factor:FactorConflict', ...
        'Novel_pmax already set by conflicting factor');
end


if chsize > chunk_property(s, 'maxchsize'),
    % need to track bigger chunks
    s = chunk_property(s, 'maxchsize', chsize);
else
    s = chunks_init(s); % initialize so CHFREQ is available
end

chfreq_g = chunk_property(s, 'chfreq');    % chunk frequency in grammatical items
chfreq_g = chfreq_g{chsize};

% how many G chunks need to be reserved?
n_novel = zeros(1, chunk_property(s, 'maxchsize'));
n_z = sum(chfreq_g(:) == 0);    % number of unsampled chunks
n_novel(chsize) = max([nov_min-n_z nov_g]);
% s = chunk_property(s, 'tgt_novel', n_novel);

n_cats = numel(tgt_cats);

% Target values for each potential chunk size, with empty vectors for those
% we don't care about (typically unigrams and trigrams).
chNovTgt = cell(n_cats, chunk_property(s, 'maxchsize'));

for i = 1:n_cats,
    chNovTgt{i, chsize} = tgt_cats(i);
end

% Determine a sensible limit for greatest fraction of novel chunks in an 
% item that could be compensated for, to maintain overall average chunk
% frequency
lenrange = item_property(s, 'lenrange');
len_min = lenrange(1);
min_chunks_per_item = max([1, len_min - chsize + 1]);
pmax = max(tgt_cats) / min_chunks_per_item;
if pmax >= 1,
    pmax = (min_chunks_per_item - 1) / min_chunks_per_item;
end
pmax_vector = zeros(1, chunk_property(s, 'maxchsize'));
pmax_vector(chsize) = pmax;

% Set chunk properties together so fwt initialization happens only once
s = chunk_property(s, 'tgt_novel', n_novel, ...
    'novel_pmax', pmax_vector);


tgt_chfreq = chunk_property(s, 'tgt_chfreq');
tgt_chfreq = tgt_chfreq{chsize};

potential_novel_chunks = (tgt_chfreq(:) == 0);

% count potential novel chunks in each item
names = itemnames(s);
n_items = length(names);
n_potential_novel_chunks = zeros(1, n_items);  % allocate space
syms = symbols(s);
for i = 1:n_items,
    chfreq = ngram(names{i}, syms, chsize);
    n_potential_novel_chunks(i) = sum(chfreq(potential_novel_chunks));
end


if nargin < 4 || isempty(catnames),
    catnames = make_level_names(fname, n_cats);
elseif ~(iscellstr(catnames) && length(catnames) == n_cats),
    error('StimSelect:RuleStr_Factor:BadCatNames', ...
        'Category names must be cell array of strings to match number of categories');
end

% Make a cell array with names of factor levels in the first row and
% corresponding factor level structures in the second row
levargs(1, :) = catnames;
for i = 1:n_cats,
    potential_items = n_potential_novel_chunks == chNovTgt{i, chsize};
    if ~any(potential_items),
        warning('StimSelect:chnov_factor:NoItems', ...
            'No items with %d novel chunks', chNovTgt{i, chsize} );
    end
    levargs{2, i} = factorlevel( potential_items, chNovTgt(i, :));
end

% Create a scaling function that puts the number of novel chunks onto a
% scale between 0 and 1, for convenience in computing goodness of fit.
chNov_scalefunc = @(x) (x + 0.125) ./ (x + 2);

f = factor_def(...
    'f_itemval_newtrain', @item_novelChunks_update, ...
    'f_scale', chNov_scalefunc, ...
    levargs{:} );

s = factor(s, fname, f );

end
