function v = testset_chnov(test, varargin)
% TESTSET_CHNOV - Compute number of novel chunks in test items
%
% TESTSET_CHNOV(S, CHFREQ, SYMS)
% TESTSET_CHNOV(S, TRAIN)
% TESTSET_CHNOV(S, TRAIN, SIZE)
% TESTSET_CHNOV(S, TRAIN, SIZE, SYMS)
%
% V = TESTSET_CHNOV(S, TRAIN, ...) first counts the frequency of chunks in
%   training items listed in TRAIN, a character array or cell array of
%   strings.  The function then counts the number of chunks of each item in S
%   that had a frequency count of zero.  This can help to
%   determine the characteristics of published stimulus sets.
%
%   SIZE specifies the chunk size (> 0) to control.  The default is 2
%   (bigrams).
%
%   SYMS is an optional string indicating the characters (symbols) of
%   interest.  The strings in TRAIN should be composed only of these
%   characters.  If SYMS is not specified, it is computed from the
%   characters contained in TRAIN.
%
% V = TESTSET_CHNOV(S, CHFREQ, SYMS) counts the number of chunks of each item
%   in S that has a frequency count of 0 in CHFREQ.  
%
%   CHFREQ is an i-dimensional matrix of frequency counts, where i is the
%   length of chunks of interest.
%
%   SYMS is a string listing the characters of interest.  All and only
%   chunks composed of these characters will be examined.
%
% SEE ALSO fwt, chnov_factor, ngram

% Copyright 2007 Todd M. Bailey
%
% This file is part of AGL StimSelect (AGLSS).
%
%     AGLSS is free software; you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation; either version 3 of the License, or
%     (at your option) any later version.
% 
%     AGLSS is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%   1.1  2007/09/17 13:15:44

if nargin < 2,
    error('Missing training items or chunk frequencies');
end

if ischar(varargin{1}) || iscellstr(varargin{1}),
    train = varargin{1};
    if nargin > 2,
        chsize = varargin{2};
    end
    if nargin < 3 || isempty(chsize),
        chsize = 2;
    end
    if nargin > 3,
        syms = varargin{3};
    end
    if nargin < 4 || isempty(syms),
        syms = unique_chars(train);
    end
    chfreq = ngram(train, syms, chsize);
else
    chfreq = varargin{1};
    syms = varargin{2};
    if isvector(chfreq),
        chsize = round(log(numel(chfreq)) / log(length(syms)));
        if length(syms)^chsize ~= numel(chfreq),
            error('Chunk frequency matrix must have n^i elements');
        end
    else
        chsize = length(size(chfreq));
    end
end

unseen = chfreq(:) == 0;

test = cellstr(test);
n_test = length(test);
v = zeros(n_test, 1);
for i = 1:n_test,
    item = test{i};
    item_chfreq = ngram(item, syms, chsize);
    v(i) = sum(item_chfreq(unseen) > 0);
end

end
