function [s, catnames, tgts] = chstr_factor(s, fname, tlevs, catnames, ...
    wts, chsize)
% CHSTR_FACTOR - Define a StimSelect chunk strength factor
%
% CHSTR_FACTOR(aglss, 'fname')
% CHSTR_FACTOR(aglss, 'fname', TGTS)
% CHSTR_FACTOR(aglss, 'fname', TGTS, N)
%   Creates a factor structure for controlling the chunk strength of AGL
%   test items.
%
%   The first argument should be an AGLSS object.  It is modified and
%   returned.
%
%   FNAME is the name of the factor.
%
%   TGTS is a vector of chunk strength target values.  
%     If any are greater than 1, TGTS values are interpreted as relative
%       chunk frequencies.  For example, the default value [1 2] specifies
%       two chunk strength categories (low and high), with high chunk
%       strength items containing chunks occurring about twice as often in
%       training items as chunks within low chunk strength items.
%     If all are in the range 0 to 1, TGTS values are interpreted as
%       absolute chunk strength targets.  Chunk strength is defined as 
%       F/(E+F), where F is the frequency of the chunk across all training
%       items, and E is the expected frequency (the average frequency
%       across all chunks).
%
%   N is a cell array of strings naming the chunk strength categories.
%   N(j) gives the name for category J.  If not supplied, or [], default
%   names are assigned, based on FNAME.
%
% CHSTR_FACTOR(aglss, 'fname', TGTS, N, W)
%   W specifies what fraction of chunks in a low chunk strength item should
%   be low frequency chunks. The same fraction of chunks in a high chunk
%   strength item should be high frequency chunks. The default value is
%   0.8.
%
% CHSTR_FACTOR(aglss, 'fname', TGTS, N, [Wlo Whi])
%   Wlo determines the fraction of chunks in a low chunk strength item that
%   should be low frequency chunks, and Whi specifies the fraction of
%   chunks in a high chunk strength item that should be high frequency
%   chunks.
%
% CHSTR_FACTOR(aglss, 'fname', TGTS, N, W, chsize)
% CHSTR_FACTOR(aglss, 'fname', TGTS, N, [Wlo Whi], chsize)
%   CHSIZE specifies the maximum chunk size (> 1) to control.
%
% [S, N] = CHSTR_FACTOR(...)
%   Also returns a cell array of strings naming the categories.  This is
%   possibly useful if N is not specified as an input argument, to get a
%   list of the default category names assigned.
%
% [S, N, T] = CHSTR_FACTOR(...)
%   Returns a vector of absolute chunk strength target values.  These may
%   be useful if TGTS are specified as relative chunk frequencies.
%
% SEE ALSO fwt, chunkstrength

% Copyright 2007 Todd M. Bailey
%
% This file is part of AGL StimSelect (AGLSS).
%
%     AGLSS is free software; you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation; either version 3 of the License, or
%     (at your option) any later version.
% 
%     AGLSS is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with this program.  If not, see <http://www.gnu.org/licenses/>.

if nargin < 6,
    chsize = chunk_property(s, 'maxchsize');
elseif chsize < 2,
    error('StimSelect:chstr_factor:ChSizeTooSmall', ...
        'Chunk strength requires chunks of size two or greater');
end

if nargin < 5 || isempty(wts),
    wlo = 0.8;
    whi = wlo;
else
    nwts = numel(wts);
    if nwts < 1 || nwts > 2,
        error('StimSelect:chstr_factor:nwts', ...
            'W parameter must be a scalar or a vector of two elements');
    end
    wlo = wts(1);
    whi = wts(end);
end

if nargin < 3 || isempty(tlevs),
    tlevs = [1 2];  % default two levels of strength, 1:2 frequency ratio
end

if all(tlevs >= 0) && all(tlevs <= 1),
    % absolute chunk strengths
    tgts = tlevs;
else
    % relative chunk frequencies
    tgts = tlevs ./ (tlevs + mean(tlevs));
end

% lowest freq chunks needed to achieve chunk strength targets from weighted
% average of low and high frequency chunks, with wlo specifying proportion
% of low frequency chunks and whi specifying proportion of high frequency
% chunks
tgt_lo = min(tgts);
tgt_hi = max(tgts);
f_lo = (tgt_lo * whi - tgt_hi * (1 - wlo)) / (whi + wlo - 1);

% highest freq chunks needed to achieve chunk strength targets
f_hi = (tgt_hi - f_lo * (1 - whi)) / whi;

% Alternative equation, should yield same result:
% f_hi = (tgt_hi * wlo - tgt_lo * (1-whi)) / (whi + wlo - 1);

% Interpolate all the target categories onto scale of chunk frequencies
fcats = interp1q([tgt_lo; tgt_hi], ...
    [f_lo; f_hi], tgts(:));

tgt_fcats = chunk_property(s, 'tgt_fcats');
if isempty(tgt_fcats),
    tgt_fcats{1} = []; % leave unigram frequencies for someone else
else
    % delete category info for longer chunks, if necessary
    tgt_fcats(chsize+1:end) = [];
end
for i= 2:chsize,
    tgt_fcats{i} = fcats';  % force to row vector
end

s = chunk_property(s, 'maxchsize', chsize, 'tgt_fcats', tgt_fcats);

n_tgts = numel(tgts);
if nargin < 4 || isempty(catnames),
    catnames = make_level_names(fname, n_tgts);
elseif ~(iscellstr(catnames) && length(catnames) == n_tgts),
    error('StimSelect:ChStr_Factor:BadCatNames', ...
        'Category names must be cell array of strings to match number of categories');
end

% Make a cell array with names of factor levels in the first row and
% corresponding factor level structures in the second row
levargs(1, :) = catnames;
for i = 1:n_tgts,
    levargs{2, i} = factorlevel(tgts(i));
end

% Create a function that will scale chunk strength values onto a normalized
% scale, so that chunk strength constraints are commensurate with
% similarity constraints for purposes of constraint satisfaction.
%
% The normalized scale ranges from 0-1, with the rescaled lower target at
% 0.25, the rescaled higher target at 0.75.
chStr_scalefunc = centileScale_func(min(tgts), 0.5, max(tgts));

v = version('-release');
if str2num(v(1:4)) > 2008,
    f = factor_def( 'f_itemval_newtrain', @item_chstr_update, ...
        'f_scale', chStr_scalefunc, ...
        levargs{:} );
else
    % legacy code
    f = factor_def( 'f_itemval_newtrain', @item_chstr_update, ...
        'f_scale', chStr_scalefunc', ...
        levargs{:} );
end

s = factor(s, fname, f );

end