function s = chunkstrength(items, ref_freqs, n_avail, syms)
% CHUNKSTRENGTH - Associative strength of item chunks to training chunks
%
% A = chunkstrength('string', R, N, S)
% A = chunkstrength(cellstr, R, N, S)
%   If the first argument I is a cell array of strings and R is a cell
%   array of chunk frequency matrices, chunkstrength(I, R, N, S) returns a
%   matrix of associative chunk strengths, with a row for each string in I
%   and a column for each chunk frequency matrix in R.
%
%   If the first argument I is a string, chunkstrength(I, R, S) returns a
%   row vector of associative chunk strengths for I.
%
%   If R is a single chunk frequency matrix, chunkstrength(I, R, S) returns
%   a column vector of associative chunk strengths, one for each string in
%   I.
%
%   N is a vector whose length matches the number of cells in R, and whose
%   values indicate the number of chunks in the corresponding cell of R
%   that are free to vary from zero frequency.  As a special case, if N(i)
%   is negative, then the number of available chunks is taken to be the
%   number of chunks with non-zero frequencies in R{i}.  If N is [], then
%   all chunks are assumed to be free to vary from zero frequency.  If R is
%   a single frequency matrix, N should be a scalar or [].  
%
%   S must be a string specifying the character set from which I is
%   composed.
%
%   The size of each dimension of each matrix in R must match the length of
%   S.
%
% chunkstrength(items, R, N)
%   If the first argument I is a structure array of items, chunkstrength(I,
%   R, N) returns a matrix of associative chunk strengths, with a row for
%   each item in I and a column for each chunk frequency matrix in R.
%
% Chunk strength is defined as F/(E+F), where F is the frequency of a chunk
% across all training items, and E is the expected frequency (the average
% frequency of chunks that are allowed in training items).

% Copyright 2007 Todd M. Bailey
%
% This file is part of AGL StimSelect (AGLSS).
%
%     AGLSS is free software; you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation; either version 3 of the License, or
%     (at your option) any later version.
% 
%     AGLSS is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with this program.  If not, see <http://www.gnu.org/licenses/>.

if isnumeric(ref_freqs),
    ref_freqs = {ref_freqs};
end

if ischar(items),
    items = {items};
end

nsizes = length(ref_freqs);
nitems = length(items);
s = 0.5 + zeros(nitems, nsizes);

if nargin < 4 || isempty(syms),
    % presumably not needed because items has a chfreq field
    lnsyms = -Inf;  % avoids warning from log(0)
    if nargin < 3,
        n_avail = [];
    end
else
    lnsyms = log(length(syms));
end

for ridx = 1:nsizes,
    if isstruct(ref_freqs),
        rfreqs = ref_freqs(ridx).freq;
        nz = ref_freqs(ridx).n_chunks - ref_freqs(ridx).n_tgt0;
    else
        rfreqs = ref_freqs{ridx};
        if isempty(n_avail),
            nz = numel(rfreqs);
        else
            nz = n_avail(ridx);
            if nz < 0,
                % count number of nonzero reference frequencies
                nz = sum(rfreqs(:) > 0);
            end
        end
    end
    rfreqs = reshape(rfreqs, [], 1);
    if any(rfreqs),
        f_expected = sum(rfreqs) ./ nz;
        % effectively divide each RFREQS by MEAN of nonzero ones
% %         rprobs = nz .* rfreqs ./ sum(rfreqs);
        if ~isstruct(items),
            chsize = round(log(numel(rfreqs)) ./ lnsyms);
        end

        for itemidx = 1:nitems,
            if isstruct(items),
                item_freqs = items(itemidx).chfreq{ridx};
            else
                item_freqs = ngram(items{itemidx}, syms, chsize);
            end
            ifreqs = reshape(item_freqs, 1, []);

            if any(ifreqs),
                f = ifreqs * rfreqs ./ sum(ifreqs);
                s(itemidx, ridx) = f ./ (f + f_expected);
% %                 s(itemidx,ridx) = ifreqs * rprobs ./ sum(ifreqs);
            end
        end
    end
end
% normalize to a scale from 0 to 1
% % s = s ./ (1 + s);

end
