function chstr = testset_chstr(test, arg1, varargin)
% TESTSET_CHSTR - Compute chunk strength in test items
%
% TESTSET_CHSTR(S, TRAIN, fsg, SIZE, ...)
% TESTSET_CHSTR(S, TRAIN, N, SYMS, SIZE, ...)
%
% TESTSET_CHSTR(S, CHFREQ, fsg, ...)
% TESTSET_CHSTR(S, CHFREQ, N, SYMS, ...)
%
% TESTSET_CHSTR(..., 'chunk_type')
%
%   This function can help to determine chunk strength characteristics of
%   published stimulus sets.  It is not used in stimulus selection.
%
% V = TESTSET_CHSTR(S, TRAIN, SIZE, ...) first counts the frequency of 
%   chunks in training items TRAIN, then computes the average strength of
%   the chunks in each test item of S.  S and TRAIN are character arrays or
%   cell arrays of strings.  The function returns a vector with one value
%   for each string in S.
%
%   Chunk strength is defined as F/(E+F), where F is the frequency of the
%   chunk, and E is the expected frequency.  Expected frequency is computed
%   as the total number of chunks of a particular size in TRAIN divided by
%   the number of allowable chunks of that size (see below).
%
%   If a finite state grammar object (fsg) is specified, the number of
%   allowable chunks of each size is determined by the grammar, so that
%   grammatical chunks are allowable and non-grammatical chunks are not.
%   [XX is this right for anchor chunks, too?]  The symbols from which
%   items are composed is also determined by the grammar.
%
%   Alternatively, the number of allowable chunks of each size and the
%   symbols allowd in items can be specified directly:
%
%   N is a vector of length SIZE-1, whose values N(i-1) specify the number of
%   allowable chunks of size i.  As a special case, if N(i-1) is negative,
%   then the number of allowable chunks is taken to be the number of chunks
%   with non-zero frequencies in TRAIN. If N is [], then all chunks are
%   allowable.  This is the same as specifying N = [s^2, s^3, ...], where s
%   is the number of symbols from which items are composed.
%
%   SYMS is a string indicating the characters (symbols) of interest.  The
%   strings in TRAIN should be composed only of these characters.  If
%   neither a finite state grammar nor SYMS is not specified (or is
%   specified as []), SYMS is computed from the characters contained in
%   TRAIN.
%
%   SIZE specifies the maximum chunk size (> 1) of interest.  
%
% V = TESTSET_CHSTR(S, CHFREQ, ...) computes chunk strength based on chunk
%   frequencies specified in CHFREQ.
%
%   CHFREQ is either a 2-dimensional matrix of bigram frequency counts, or
%   a cell array of i-dimensional frequency matrices of the sort returned
%   by ngram(), where i ranges over chunk lengths greater than 1.
%
%   N is as described above.  
%
%   SYMS is a string listing the characters of interest.  All and only
%   chunks composed of these characters will be examined.
%
% V = TESTSET_CHSTR(..., 'chunk_type')
%   'chunk_type' should be one of 'all', 'anchor', 'initial', or 'final',
%   to indicate grammaticality of all chunks, anchor chunks (those
%   occurring at the beginning or end of grammatical strings), initial
%   chunks only, or final chunks only.  The default is 'all'.
%
% SEE ALSO fwt, chstr_factor, ngram

% Copyright 2007 Todd M. Bailey
%
% This file is part of AGL StimSelect (AGLSS).
%
%     AGLSS is free software; you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation; either version 3 of the License, or
%     (at your option) any later version.
% 
%     AGLSS is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%   1.1  2007/09/17 13:15:44

if nargin < 2,
    error('Missing training items or chunk frequencies');
end

args = make_stack('cells');
args.init(varargin{:});

f = args.pop();
if isa(f, 'fsg'),
    syms = symbols(f);
else
    n_nov = f;
    syms = args.pop();
end

if ischar(arg1) || iscellstr(arg1),
    % TRAIN items are specified
    train = arg1;
    chsize = args.pop();
    if isempty(chsize),
        error('Chunk size must be specified');
    end
    
    if isempty(syms),
        syms = unique_chars(train);
    end
    chfreq = ngram(train, syms, 2:chsize);
else
    % CHFREQ is specified
    chfreq = arg1;

    if iscell(chfreq),
        chsize = numel(chfreq) + 1;
    else
        % chfreq should be a bigram matrix
        if isvector(chfreq),
            % assume it has been flattened to a vector
            chsize = 2;
        else
            chsize = length(size(chfreq));
            if chsize ~= 2,
                error('Frequency counts must be in cell array or 2-D matrix');
            end
        end
    end

    if isempty(syms),
        error('Cannot determine symbols without training items');
    end
end

type = args.pop();  % chunk type
if isempty(type),
    type = 'all';
end

if chsize < 2,
    error('StimSelect:testset_chstr:ChSizeTooSmall', ...
        'Chunk strength requires chunks of size two or greater');
end

type = lower(type);
if ~strcmp(type, 'all');
    warning('Anchor chunks not yet implemented');
end

if isa(f, 'fsg'),
    n_nov = zeros(1, chsize-1);
    for i = 2:chsize,
        gcx = grammatical_chunks(f, i);
        n_nov(i-1) = sum(gcx(:));
    end
end

chstr = chunkstrength(test, chfreq, n_nov, syms);

chstr = mean(chstr, 2);

end
