function f = factor_def(varargin)
% FACTOR_DEF - Define AGL test factor and levels
%
% Syntax
% FACTOR_DEF(ITEMS_OK, 'func_name1', F1, ..., 'level1', factorlevel1, ...)
% FACTOR_DEF('func_name1', F1, ..., 'level1', factorlevel1, ...)
%
% Description
%   Creates a test factor structure with the specified factor levels and
%   default functions 
%
%   If the first argument is a logical vector, it is used as the default
%   for specifying which items to consider adding to test sets with the
%   various levels of this factor.  Omitting this argument or specifying it
%   as logical([]) places no restrictions on the items to consider.
%
%   Function name-value pairs are as in FACTORLEVEL, but the additional
%   function name 'f_itemval_newtrain' is recognized.  If a particular
%   function name is specified for a factor level as well as the factor as
%   a whole, the one specified for the factor level is called for that
%   level, and the one specified for the factor as a whole is used only as
%   a default for levels in which the function is not specified to
%   FACTORLEVEL.
%       'f_itemval_newtrain'
%           Updates an item's value for the relevant factor to take account
%           of new items being added to the training set.  The function is
%           called as F(M, 'factor', LEVNAMES, TGT, F_SCALE, T, I) to update
%           items M with respect to the given factor.  T is the current
%           training set, I is a new item(s) that has just been added to the
%           training set, TGT is the target value for the specified factor
%           and level, and F_SCALE is the scaling function assigned to that
%           factor and level.

% Copyright 2007 Todd M. Bailey
%
% This file is part of AGL StimSelect (AGLSS).
%
%     AGLSS is free software; you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation; either version 3 of the License, or
%     (at your option) any later version.
% 
%     AGLSS is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with this program.  If not, see <http://www.gnu.org/licenses/>.

f = struct(...
    'levelnames', {{}} );

isavail = [];

valid_fnames = {...
    'f_itemval_newtrain', ...
    'f_itemfit', ...
    'f_exclude', ...
    'f_scale', ...
    'f_testset_newtrain', ...
    'f_testset_newtest' };  % NB not used by choose_items; obsolete?

funcs = struct;
for i = 1:length(valid_fnames),
    funcs.(valid_fnames{i}) = [];
end

% check whether first argument is logical vector indicating relevant items
if length(varargin) > 0 && islogical(varargin{1}),
    isavail = varargin{1};
    varargin(1) = [];
end


while ~isempty(varargin),
    if length(varargin) < 2,
        error('Factor level names and structures must come in pairs');
    end
    if ~ischar(varargin{1}),
        error('Factor level names must be strings');
    end
    levname = varargin{1};
    flev = varargin{2};
    varargin(1:2) = [];
    
    % is it a function name?
    f_idx = strmatch(lower(levname), valid_fnames, 'exact');
    if ~isempty(f_idx),
        funcs.(levname) = flev;
    else
        % interpret level name-value pair
        if isempty(flev) && ~isempty(isavail),
            flev = struct('excluded_items', ~isavail);
        end
        if ~isempty(flev),
            if isempty(flev.excluded_items),
                flev.excluded_items = ~isavail;
            end
            if isfield(flev, 'tgt'),
                % fill in default functions if necessary
                for i = 1:length(valid_fnames),
                    field = valid_fnames{i};
                    if isfield(flev, field) && isempty(flev.(field)),
                        flev.(field) = funcs.(field);
                    end
                end
            end
        end
        f.(levname) = flev;
        f.levelnames{end+1} = levname;
    end
end

if ~isempty(funcs.f_itemval_newtrain),
    f.f_itemval_newtrain = funcs.f_itemval_newtrain;
end

end
