function lev = factorlevel(varargin)
% FACTORLEVEL - Define a level of an AGL stimulus factor
%
% Syntax
%   FACTORLEVEL
%   FACTORLEVEL(ITEMS_OK)
%   FACTORLEVEL(ITEMS_OK, TGT, 'func_name1', F1, ...)
%   FACTORLEVEL(TGT, 'func_name1', F1, ...)
%   FACTORLEVEL(..., W)
%
% FACTORLEVEL (obsolete?) returns a factor level without target values,
% which will not be evaluated in determining fitness values for candidate
% test items. This is useful for factor levels that are determined by prior
% selection of appropriate candidate items (e.g. 'G' and 'NG' levels of a
% grammaticality factor, or 'Old' and 'New' levels of an item familiarity
% factor).
%
% FACTORLEVEL(ITEMS_OK, TGT, 'func_name1', F1, ...) 
%   Returns a factor level structure with target value TGT, in which the
%   only items that will be considered for inclusion in the relevant set
%   are those indicated by TRUE values in corresponding locations in the
%   logical vector ITEMS_OK.  
%
%   Function name-value pairs allow you to specify functions to perform
%   various dynamic update operations during stimulus item selection.  The
%   recognized function names are:
%       'f_itemfit'
%           Called as F(M, 'factor', 'level', TGT, F_SCALE, T, I) to update
%           items M with respect to the given factor.  T is the current
%           training set, I is a new item(s) that has just been added to the
%           training set, TGT is the target value for the specified factor
%           and level, and F_SCALE is the scaling function assigned to that
%           factor and level.
%
%       'f_exclude'     Exclude certain items from a test set
%           Called as F(...)  (actually, not just a function)
%       'f_scale'
%       'f_newtrain'
%       'f_newtest'
%
%   The user must ensure that G has the same length as the set of potential
%   items.
%
% FACTORLEVEL(TGT, ...) 
%   If the first argument is not of type logical, all items are eligible.
%
% FACTORLEVEL(..., W)
%   Specifies a weighting factor to be applied to the fitness values
%   associated with this factor and level.  W defaults to 1 if not
%   specified.
%
% Items being evaluated as candidate test items will be expected to have a
% field corresponding to this factor, with fields .val and .fits.(G) (for
% example, item I should allow references to I.sim.val and
% I.sim.fits.lowsim if level 'lowsim' is defined for factor 'sim').
%
% f_newT, f_newS, and f_xclud default to [], i.e. no functions.

% Copyright 2007 Todd M. Bailey
%
% This file is part of AGL StimSelect (AGLSS).
%
%     AGLSS is free software; you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation; either version 3 of the License, or
%     (at your option) any later version.
% 
%     AGLSS is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with this program.  If not, see <http://www.gnu.org/licenses/>.

isavail = [];
tgt = [];
wt = 1;

valid_fnames = {...
    'f_itemfit', ...
    'f_exclude', ...
    'f_scale', ...
    'f_testset_newtrain', ...
    'f_testset_newtest' };  % NB not used by choose_items; obsolete?

funcs = struct;
for i = 1:length(valid_fnames),
    funcs.(valid_fnames{i}) = [];
end

% check whether first argument is logical vector indicating relevant items
if length(varargin) > 0 && islogical(varargin{1}),
    isavail = reshape(varargin{1}, 1, []);  % force to row vector
    varargin(1) = [];
end


% now, first argument should be target value
if length(varargin) > 0,
    tgt = varargin{1};
    varargin(1) = [];
end

while ~isempty(varargin),
    if ischar(varargin{1}),
        % interpret a function name - function handle pair
        i = strmatch(lower(varargin{1}), valid_fnames, 'exact');
        if isempty(i),
            error('Unexpected function name ''%s''', varargin{1});
        end
        fname = valid_fnames{i};
        if length(varargin) < 2,
            error('Function names and handles must occur in pairs');
        end
        func = varargin{2};
        varargin(1:2) = [];
        funcs.(fname) = func;
    else
        % constraint weighting
        if length(varargin) > 1,
            error('Further arguments not allowed after constraint weighting');
        end
        wt = varargin{1};
        varargin(1) = [];
    end
end
        
if isempty(tgt),
    if isempty(isavail) && ~islogical(isavail) && isempty(funcs.f_exclude),
        lev = [];
    else
        lev = struct('excluded_items', ~isavail);
        if ~isempty(funcs.f_exclude),
            lev.f_exclude = funcs.f_exclude;
        end
    end
    return
end
    
lev = struct(...
    'excluded_items', ~isavail, ...
    'tgt', {tgt}, ...
    'wt', {wt} );

for i = 1:length(valid_fnames),
    lev.(valid_fnames{i}) = funcs.(valid_fnames{i});
end

end