function [s, catnames] = fam_factor( s, fname, catnames )
% FAM_FACTOR - Define a StimSelect familiarity factor
%
% FAM_FACTOR( aglss, 'fname' )
% FAM_FACTOR( aglss, 'fname', N )
%   Creates a factor structure for controlling the familiarity of AGL
%   test items, so that some sets of test items include familiar training
%   items, and some include only new items that did not appear in the
%   training set.
%
%   The first argument should be an AGLSS object.  It is modified and
%   returned.
%
%   FNAME is the name of the factor.
%
%   N is an optional cell array of strings naming the two familiarity
%   categories.  The default names are 'Old' and 'New'.
%
% [S,N] = GRAM_FACTOR(...)
%   Returns a cell array of strings naming the categories.  This is
%   possibly useful if N is not specified as an input argument, to get a
%   list of the default category names assigned.
%
% SEE ALSO: def_factor

% Copyright 2013 Todd M. Bailey
%
% This file is part of AGL StimSelect (AGLSS).
%
%     AGLSS is free software; you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation; either version 3 of the License, or
%     (at your option) any later version.
% 
%     AGLSS is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with this program.  If not, see <http://www.gnu.org/licenses/>.

if ~isa(s, 'aglss'),
    error('StimSelect:Gram_Factor:Type', ...
        'First argument must be AGLSS object');
end

if nargin < 3,
    catnames = {};
elseif ~(iscellstr(catnames) && length(catnames) == 2),
    error('StimSelect:FamFactor:BadCatNames', ...
        'Category names must be cell array of two strings');
end

if isempty(catnames),
    catnames = {'Old', 'New'};
end

% Make a cell array with names of factor levels in the first row and
% corresponding factor level structures in the second row
levargs(1, :) = catnames;
levargs{2, 1} = factorlevel(true(0), [], 'f_exclude', @fam_exclude_new);    %Old
levargs{2, 2} = factorlevel(true(0), [], 'f_exclude', @fam_exclude_old);    %New

% Need to update status when items are added to training
% % f = factor_def('f_itemval_newtrain', @item_fam_update, levargs{:});
f = factor_def(levargs{:});

s = factor(s, fname, f );

end

function [exclude, reschedule] = fam_exclude_old(items, train, tset)
% fam_exclude_old - dynamically exclude old (training) items
%
% [xn, r] = fam_exclude_old(I, T, S) ignores items I and test set S, but
% returns a vector XN of logical values indicating which items are included
% in the training set.

% Copyright 2013 Todd M. Bailey
%
% This file is part of AGL StimSelect (AGLSS).
%
%     AGLSS is free software; you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation; either version 3 of the License, or
%     (at your option) any later version.
% 
%     AGLSS is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with this program.  If not, see <http://www.gnu.org/licenses/>.

exclude = train.items;

reschedule = true;  % must update it in case training items are added

end

function [exclude, reschedule] = fam_exclude_new(items, train, tset)
% fam_exclude_new - dynamically exclude new (non-training) items
%
% [xn, r] = fam_exclude_new(I, T, S) ignores items I and test set S, but
% returns a vector XN of logical values indicating which items are not
% included in the training set.

% Copyright 2013 Todd M. Bailey
%
% This file is part of AGL StimSelect (AGLSS).
%
%     AGLSS is free software; you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation; either version 3 of the License, or
%     (at your option) any later version.
% 
%     AGLSS is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with this program.  If not, see <http://www.gnu.org/licenses/>.

exclude = ~train.items;

reschedule = true;  % must update it in case training items are added

end
