function [s, catnames] = gram_factor( s, fname, varargin )
% GRAM_FACTOR - Define a StimSelect grammaticality factor
%
% GRAM_FACTOR( aglss, 'fname' )
% GRAM_FACTOR( aglss, 'fname', N )
% GRAM_FACTOR( aglss, 'fname', grammar )
% GRAM_FACTOR( aglss, 'fname', grammar, N )
%   Creates a factor structure for controlling the grammaticality of AGL
%   test items, so that some sets of test items are grammatical, and some
%   are not.
%
%   The first argument should be an AGLSS object.  It is modified and
%   returned.
%
%   FNAME is the name of the factor.
%
%   N is an optional cell array of strings naming the two grammaticality
%   categories.  The default names are 'G' and 'NG', for grammatical and
%   non-grammatical items, respectively.
%
%   grammar is an optional FSG object specifying the grammar to use.  The
%   default is to use the grammar of aglss. A different grammar can be used
%   in combination with the default one to achieve greater control over
%   test items that violate the default grammar.
%
% [S,N] = GRAM_FACTOR(...)
%   Returns a cell array of strings naming the categories.  This is
%   possibly useful if N is not specified as an input argument, to get a
%   list of the default category names assigned.
%
% The grammatical items of the AGLSS object are determined by calling
% ITEM_PROPERTY(aglss, 'isg').
%
% SEE ALSO: item_property, cat_factor

% Copyright 2007 Todd M. Bailey
%
% This file is part of AGL StimSelect (AGLSS).
%
%     AGLSS is free software; you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation; either version 3 of the License, or
%     (at your option) any later version.
% 
%     AGLSS is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with this program.  If not, see <http://www.gnu.org/licenses/>.

if ~isa(s, 'aglss'),
    error('StimSelect:Gram_Factor:Type', ...
        'First argument must be AGLSS object');
end

grammar = [];
catnames = {};
if nargin >= 3,
    if isa(varargin{1}, 'fsg'),
        grammar = varargin{1};
        % grammarname = inputname(nargin-size(varargin,2)+1);
    elseif nargin > 3,
        error('StimSelect:GramFactor:BadGrammar', ...
            'Grammar must be object of type FSG');
    end
    if ~isa(varargin{1}, 'fsg') || nargin > 3,
        catnames = varargin{end};
        if ~(iscellstr(catnames) && length(catnames) == 2),
            error('StimSelect:GramFactor:BadCatNames', ...
                'Category names must be cell array of two strings');
        end
    end
end

if isempty(catnames),
    catnames = {'G', 'NG'};
end

% Identify a priori which potential items are eligible for each level of
% this factor
if isempty(grammar),
    isG = item_property(s, 'isg');
else
    isG = isgrammatical(grammar, itemnames(s));
end

% Identify a priori which potential items are eligible for each level of
% this factor
if size(isG, 2) > 1,
    isG = isG(:);   % force column vector
end
s = cat_factor(s, fname, [isG, ~isG], catnames);

% % f = factor_def('G', factorlevel(isG), 'NG', factorlevel(~isG));
% % s = factor(s, fname, f );

end
