function [s, catnames, tgt_cats] = rulestr_factor(s, fname, ...
    tgt_cats, catnames, n_reserved, chsize)
% RULESTR_FACTOR - Define a StimSelect rule strength factor
%
% RULESTR_FACTOR(aglss, 'fname', cats, NAMES, m, chsize)
% RULESTR_FACTOR(aglss, 'fname', cats, NAMES, [n m], chsize)
%   Returns a factor structure for controlling the rule strength of AGL
%   test items.  Rule strength is the fraction of a test item's chunks that
%   are NOT novel (averaged across all chunks from bigrams up to a
%   particular length).  In other words, the fraction of chunks in a test
%   item that are consistent with (the chunks appearing in) training items.
%
%   The first argument should be an AGLSS object.  It is modified and
%   returned.
%
%   FNAME is the name to give the factor (e.g. 'RuleStr').
%
%   CATS is a vector of rule strength target values, specified as fraction
%   of an item's chunks that should be novel.  The length of CATS
%   determines the number of levels of the rule strength factor.  The
%   default value of CATS is [0.75, 1], creating a rule strength factor
%   with two levels (low and high rule strength) with 75% or 100% familiar
%   chunks, respectively.
%
%   NAMES is a cell array of strings naming the chunk strength categories.
%   NAMES(j) gives the name for category J.  If not supplied, or [],
%   default names are assigned, based on FNAME.
%   
%   M specifies the minimum number of grammatical chunks to exclude from
%   training items, reserving them to be used as novel chunks in test
%   items.  This allows chunk novelty to be combined factorially with
%   grammaticality.  The default value for M is 2.  
%
%   [N M] specifies the minimum number of chunks (whether grammatical or
%   not) to exclude from training items, and then the minimum number of
%   grammatical chunks to exclude.  In particular, M can be set to 0 if
%   grammaticality is not going to be used as a stimulus factor.
%   Grammatical chunks are excluded only as a last resort, that is, if N is
%   greater than the number of non-grammatical chunks, or if M > 0.
%
%   CHSIZE specifies the maximum chunk size (> 1) to control.  If chunks
%   are to be excluded from training items, they will always be chunks of
%   size CHSIZE.
%
% [S, NAMES] = RULESTR_FACTOR(...)
%   Also returns a cell array of strings naming the categories.  This is
%   possibly useful if NAMES is not specified as an input argument, to get
%   a list of the default category names assigned.
%
% [S, N, T] = RULESTR_FACTOR(...)
%   Returns a vector of rule strength target values.  These may
%   be useful if TGT_CATS are not specified as an input argument.
%
% SEE ALSO chnov_factor, fwt, chstr_factor, ngram

% Copyright 2007 Todd M. Bailey
%
% This file is part of AGL StimSelect (AGLSS).
%
%     AGLSS is free software; you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation; either version 3 of the License, or
%     (at your option) any later version.
% 
%     AGLSS is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with this program.  If not, see <http://www.gnu.org/licenses/>.

if nargin < 6,
    chsize = chunk_property(s, 'maxchsize');
    s = chunks_init(s); % initialize so that CHFREQ is available
else
    if chsize < 2,
        error('StimSelect:rulestr_factor:ChSizeTooSmall', ...
            'Rule strength requires chunks of size two or greater');
    end
    s = chunk_property(s, 'maxchsize', chsize);
end

if nargin < 5,
    n_reserved = [];
end
if isempty(n_reserved),
    nov_g = 2;
elseif isscalar(n_reserved),
    nov_g = n_reserved;
else
    nov_g = n_reserved(2);
end
if length(n_reserved) < 2,
    nov_min = nov_g;
else
    nov_min = max([nov_g, n_reserved(1)]);
end

if nargin < 3 || isempty(tgt_cats),
    tgt_cats = [0.75, 1];
end
n_tgts = numel(tgt_cats);


if ~isempty(chunk_property(s, 'novel_pmax')),
    error('StimSelect:rulestr_factor:FactorConflict', ...
        'Novel_pmax already set by conflicting factor');
end

chfreq_g = chunk_property(s, 'chfreq');    % chunk frequency in grammatical items
chfreq_g = chfreq_g{chsize};

% how many G chunks need to be reserved?
n_novel = zeros(1, chunk_property(s, 'maxchsize'));
n_z = sum(chfreq_g(:) == 0);    % number of NG chunks
n_novel(chsize) = max([nov_min-n_z nov_g]);
% s = chunk_property(s, 'tgt_novel', n_novel);

% Determine a sensible limit for greatest fraction of novel chunks in an 
% item that could be compensated for, to maintain overall average chunk
% frequency
lenrange = item_property(s, 'lenrange');
len_min = lenrange(1);
min_chunks_per_item = max([1, len_min - chsize + 1]);
pmax = 1 - min(tgt_cats);
if pmax >= 1,
    pmax = (min_chunks_per_item - 1) / min_chunks_per_item;
end
pmax_vector = zeros(1, chunk_property(s, 'maxchsize'));
pmax_vector(chsize) = pmax;

% Set chunk properties together so fwt initialization happens only once
s = chunk_property(s, 'tgt_novel', n_novel, ...
    'novel_pmax', pmax_vector);

if nargin < 4 || isempty(catnames),
    catnames = make_level_names(fname, n_tgts);
elseif ~(iscellstr(catnames) && length(catnames) == n_tgts),
    error('StimSelect:RuleStr_Factor:BadCatNames', ...
        'Category names must be cell array of strings to match number of categories');
end

% Make a cell array with names of factor levels in the first row and
% corresponding factor level structures in the second row
levargs(1, :) = catnames;
for i = 1:n_tgts,
    levargs{2, i} = factorlevel(tgt_cats(i));
end

% % tgt_cats = unique(tgt_cats);

% Create a scaling function that puts the fraction of novel chunks onto a
% sensible scale between 0 and 1, for convenience in computing goodness of
% fit.

t_min = min(tgt_cats);
t_max = max(tgt_cats);

if t_min == 0,
    x_scale = [0];
    y_scale = [0.10];
else
    x_scale = [0; t_min];
    y_scale = [0; 0.25];
end

if t_max == 1,
    x_scale = [x_scale; 1];
    y_scale = [y_scale; 0.90];
else
    x_scale = [x_scale; t_max; 1];
    y_scale = [y_scale; 0.75; 1];
end

ruleStr_scalefunc = @(x) interp1q(x_scale, ...
    y_scale, x(:));

f = factor_def(...
    'f_itemval_newtrain', @item_rulestr_update, ...
    'f_scale', ruleStr_scalefunc, ...
    levargs{:} );

s = factor(s, fname, f );

end