function s = rulestrength(item_freqs, ref_freqs, subset)
% RULESTRENGTH - Rule strength of items based on chunk familiarity
%
% RULESTRENGTH(I, R, IDX)
%   Computes rule strength of items, that is, the fraction of an item's
%   chunks whose reference frequency in R is greater than zero.  Usually R
%   contains frequency counts of the chunks in a set of AGL training items.
%
% If I is a matrix, each row should contain chunk frequency counts for a
% different item.  R should be a numeric vector or matrix whose number of
% elements matches the number of columns in I, containing reference chunk
% frequencies (usually the frequency of chunks of a particular size in a
% set of training items).
%
% If I is a cell array, there should be one row for each item, and each
% cell should contain chunk frequency counts for that item.  Different
% columns of cells may contain frequency counts for chunks of different
% sizes.  R should be a cell vector whose length matches the number of
% columns of I, with each cell containing a vector or matrix of reference
% frequencies.  Alternatively, if I contains a single column, R may be a
% vector of reference frequencies.
%
% IDX is an optional vector of subscripts indicating which columns of I and
% R to use.  If not specified, or [], all columns are used.
%
% The return value will be a column vector of rule strengths, one for each
% row of I.

% Copyright 2007 Todd M. Bailey
%
% This file is part of AGL StimSelect (AGLSS).
%
%     AGLSS is free software; you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation; either version 3 of the License, or
%     (at your option) any later version.
% 
%     AGLSS is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with this program.  If not, see <http://www.gnu.org/licenses/>.

if nargin < 3,
    subset = [];
end

if isnumeric(ref_freqs),
    ref_freqs = {ref_freqs};
end

if ~isempty(subset),
    ref_freqs = ref_freqs(:, subset);
    item_freqs = item_freqs(:, subset);
end

n_items = size(item_freqs, 1);

n_sizes = length(ref_freqs);

% identify positive-frequency reference chunks
nz = cell(1, n_sizes);
for ridx = 1:n_sizes,
    nz{ridx} = ref_freqs{ridx}(:) > 0;
% %     if isstruct(ref_freqs),
% %         rfreqs = ref_freqs(ridx).freq;
% %     else
% %         rfreqs = ref_freqs{ridx};
% %     end
% %     nz{ridx} = (rfreqs(:) > 0);
end

% count total chunks, and total familiar chunks for each item
n_fam = zeros(n_items, 1);
n_chunks = zeros(n_items, 1);
for itemidx = 1:n_items,
    for subsetnum = 1:n_sizes,
        if isnumeric(item_freqs),
            ifreqs = item_freqs(itemidx, :);
        else
            ifreqs = item_freqs{itemidx, subsetnum}(:);
        end
        
        n_fam(itemidx) = n_fam(itemidx) + sum(ifreqs(nz{subsetnum}));
        n_chunks(itemidx) = n_chunks(itemidx) + sum(ifreqs);
    end
end
s = n_fam ./ max(n_chunks, 1);  % avoid divide by 0

end
