function rulestr = testset_rulestr(test, arg1, syms, chsize)
% TESTSET_RULESTR - Compute rule strength for a set of test items
%
% TESTSET_RULESTR(S, TRAIN, SYMS, SIZE)
% TESTSET_RULESTR(S, CHFREQ, SYMS)
%
%   This function can help to determine chunk strength characteristics of
%   published stimulus sets.  It is not used in stimulus selection.
%
% V = TESTSET_RULESTR(S, TRAIN, 'symbols', SIZE) computes rule strength for
%   each test item in S.  Rule strength is the fraction of an item's chunks
%   whose training frequency is greater than zero.  
%
%   S specifies a test items.  It can be a character array or cell array of
%   strings.
%
%   TRAIN specifies a list of training items directly.  It can be a
%   character array or cell array of strings.
%
%   'symbols' is a string specifying the symbols from which items are
%   composed.  If given as [], the unique characters of TRAIN are used.
%
%   SIZE is a vector specifying the maximum chunk size (> 1) of interest.
%
% V = TESTSET_RULESTR(S, R, 'symbols') computes rule strength of test items
%   based on reference frequencies R.  R should be a cell vector whose
%   length is the maximum chunk size of interest minus 1.  Each cell R{i}
%   should contain a vector or matrix of frequencies for chunks of size
%   i-1.

% Copyright 2007 Todd M. Bailey
%
% This file is part of AGL StimSelect (AGLSS).
%
%     AGLSS is free software; you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation; either version 3 of the License, or
%     (at your option) any later version.
% 
%     AGLSS is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%   1.1  2007/09/17 13:15:44


if nargin < 2,
    error('Missing training items or chunk frequencies');
end

if ischar(arg1) || iscellstr(arg1),
    % TRAIN items are specified
    train = arg1;
    if isempty(syms),
        syms = unique_chars(train);
    end
    if nargin < 4 || isempty(chsize),
        error('Chunk size must be specified');
    elseif chsize < 2,
        error('Chunk size must be greater than 1');
    end
    chfreq = ngram(train, syms, 2:chsize);
else
    % CHFREQ is specified
    chfreq = arg1;

    if iscell(chfreq),
        chsize = numel(chfreq) + 1;
    else
        % chfreq should be a bigram matrix
        if isvector(chfreq),
            % assume it has been flattened to a vector
            chsize = 2;
        else
            chsize = length(size(chfreq));
            if chsize ~= 2,
                error('Frequency counts must be in cell array or 2-D matrix');
            end
        end
    end

    if nargin < 3 || isempty(syms),
        error('Cannot determine symbols without training items');
    end
    if nargin > 3,
        warning('Ignoring superfluous chunk size argument');
    end
end

test = cellstr(test);
n_test = length(test);

itemfreqs = cell(n_test, chsize-1);
for i = 1:n_test,
    f = ngram(test{i}, syms, 2:chsize);
    if iscell(f),
        itemfreqs(i, :) = ngram(test{i}, syms, 2:chsize);
    else
        % chsize == 2
        itemfreqs(i) = {f};
    end
end

rulestr = rulestrength(itemfreqs, chfreq);

end
