function s = EditSim(s1, s2, varargin)
% EditSim - Normalized similarity between two strings.
%
% Normalized similarity is computed by dividing the edit distance between
% two strings by the maximum possible edit distance for strings of the
% given lengths, and subtracting the result from 1.  This produces a
% similarity value between 0 (maximally different) and 1 (identical).
%
% EditSim(s1, s2) 
% Sets the cost of character indels and replacements to the value 1.
%
% EditSim(s1, s2, Dcost, Icost, Rcost) 
% Sets the cost of character deletions, insertions, and replacements to
% Dcost, Icost, and Rcost, respectively.  If Dcost and Icost differ, the
% directionality is for changing s1 into s2.
%
% If either s1 or s2 (or both) are cell arrays of strings, EditSim returns
% a matrix of pairwise similarities between the items in s1 and s2.  The
% similarity matrix will have a row for each item in s1 and a column for each
% item in s2.

% Copyright 2007 Todd M. Bailey
%
% This file is part of AGL StimSelect (AGLSS).
%
%     AGLSS is free software; you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation; either version 3 of the License, or
%     (at your option) any later version.
% 
%     AGLSS is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with this program.  If not, see <http://www.gnu.org/licenses/>.

%Deal with cell arrays of strings if necessary.
%Uses one level of recursion.
if iscell(s1) || iscell(s2),
    if ~iscell(s1),
        s1 = {s1};
    elseif ~iscell(s2),
        s2 = {s2};
    end
    n1 = length(s1);
    n2 = length(s2);
    s = zeros(n1,n2);
    for i=1:n1,
        for j=1:n2,
            s(i,j) = EditSim(s1{i}, s2{j}, varargin{:});
        end
    end
    return
end

if ~isempty(varargin)
   if length(varargin) ~= 3
      error('Usage is: EditSim(''string1'',''string2'',DeleteCost,InsertCost,ReplaceCost)');
   end;
   DelCost = varargin{1};
   InsCost = varargin{2};
   ReplCost = varargin{3};
else
   DelCost = 1;
   InsCost = 1;
   ReplCost = 1;
end;

n1=length(s1);
n2=length(s2);
if n1 >= n2,
    max_d = (n1-n2)*DelCost + n2*ReplCost;
else
    max_d = (n2-n1)*InsCost + n1*ReplCost;
end
% Check whether it's cheaper to do all indels
max_d = min([max_d, (n1*DelCost + n2*InsCost)]);

if max_d == 0,
    s = 1;
    return
end

d = EditDist(s1, s2, DelCost, InsCost, ReplCost);
s = 1 - d/max_d;

end