function [s, catnames, tgts] = sim_factor(s, fname, tlevs, catnames, ...
    s_func, s_parms, sim_scalefunc)
% SIM_FACTOR - Define a StimSelect similarity factor
%
% SIM_FACTOR(aglss, 'fname')
% SIM_FACTOR(aglss, 'fname', TGTS)
% SIM_FACTOR(aglss, 'fname', TGTS, NAMES)
% SIM_FACTOR(aglss, 'fname', TGTS, NAMES, F)
% SIM_FACTOR(aglss, 'fname', TGTS, NAMES, F, FPARMS)
% SIM_FACTOR(aglss, 'fname', TGTS, NAMES, F, FPARMS, SCALE_FUNC)
%   Creates a factor structure for controlling the similarity of AGL
%   test items to training items.
%
%   The first argument should be an AGLSS object.  It is modified and
%   returned.
%
%   FNAME is the name of the factor.
%
%   TGTS is a vector of similarity target values.  If SCALE_FUNC is not
%   specified, target values are specified as percentiles which refer to
%   the pairwise similarities of all grammatical items to each other. The
%   default is [25 75], which uses the 25th and 75th percentile
%   similarities to form a 2-level similarity factor (i.e. low similarity
%   and high similarity).  If SCALE_FUNC is specified, then target values
%   are raw, unscaled similarity values (i.e. values of the sort returned
%   by the similarity function F).
%
%   F is a function handle identifying the desired similarity function.  It
%   should be a function that takes two arguments S1 and S2, and may also
%   take additional arguments specified in FPARMS.  S1 and S2 may be either
%   strings or cell arrays of strings, and F must be able to handle both.
%   If both are strings, F should return a scalar similarity value.  If
%   either S1 or S2 is a cell array of strings, F should return a matrix of
%   similarity values, with a row for each string in S1 and a column for
%   each string in S2.  The default is @EditSim, which will be called as
%   EditSim(string1, string2, 0.7, 0.7, 1), that is, with insertion and
%   deletion costs of 0.7 and substitution cost of 1 (see Hahn & Bailey,
%   2005).  These default costs can be changed by specifying the FPARMS
%   parameter.
%
%   FPARMS is a cell array of extra arguments to be passed to F, which will
%   be called as F('string1', 'string2', FPARMS{:}).  As a special case, if
%   F is not specified, FPARMS defaults to {0.7, 0.7, 1} as described
%   above.  If F is specified but FPARMS is not, FPARMS defaults to {},
%   which passes no extra arguments to F.
%
%   SFUNC is a function handle that specifies a scaling function to use
%   as an alternative to the default scaling function based on similarities
%   among grammatical items.  If SFUNC is specified, TGTS must be a vector
%   of unscaled similarity values.  Those values will define the levels of
%   the similarity factor.
%
% SEE ALSO EditSim, chstr_factor

% Copyright 2007 Todd M. Bailey
%
% This file is part of AGL StimSelect (AGLSS).
%
%     AGLSS is free software; you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation; either version 3 of the License, or
%     (at your option) any later version.
% 
%     AGLSS is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with this program.  If not, see <http://www.gnu.org/licenses/>.

if nargin < 7,
    sim_scalefunc = [];
end

if nargin < 5 || isempty(s_func),
    s_func = @EditSim;
    ReplCost = 1;
    IndelCost = 0.7 * ReplCost;
    s_parms = {IndelCost, IndelCost, ReplCost};
elseif nargin < 6 || isempty(s_parms),
    s_parms = {};
end

% Embed the similarity function and parameters in an update function for
% items.
item_sim_update_handle = item_sim_update_func(s_func, s_parms{:});


if isempty(sim_scalefunc),
    % Determine high and low target similarity scores.
    %
    % An overall similarity score for a test item is computed as the
    % average similarity between it and the set of training items.
    %
    % Default target categories aim at the 75th and 25th percentile
    % pairwise similarity scores of all grammatical items.
    strings = itemnames(s, item_property(s, 'isg'));
    n_g = length(strings);
    S = eye(n_g);
    for i = 1:n_g-1,
        for j = i+1:n_g,
            S(i,j) = s_func(strings{i}, strings{j}, s_parms{:});
        end
    end
    S = S(triu(true(n_g), 1));  % values of S above the diagonal

    if nargin < 3 || isempty(tlevs),
        % Default to two levels of the similarity factor, at the 25th and
        % 75th percentile values observed among STRINGS
        tlevs = [25 75];
    elseif all(tlevs <= 1 & tlevs >= 0),
            warning('StimSelect:sim_factor:fractional_targets', ...
                ['Target levels appear to be fractions rather than ', ...
                'percentiles'] );
    end
    tgts = prctile(S, tlevs);

    % Create a function that will scale similarity values onto a normalized
    % scale, so that similarity constraints are commensurate with
    % chunk constraints, etc., for purposes of constraint satisfaction.
    %
    % The normalized scale ranges from 0-1, with the rescaled lower target at
    % 0.25, the rescaled higher target at 0.75, and the rescaled average at
    % 0.50.
    if numel(tgts) <= 3,
        ptiles = tgts;
    else
        ptiles = [tgts(1), (tgts(1)+tgts(end))/2, tgts(end)];
    end
    sim_stats = num2cell( ptiles );
    sim_scalefunc = centileScale_func(sim_stats{:});
else
    if any(tlevs < 0 | tlevs > 1),
        error('StimSelect:sim_factor:outsize_targets', ...
            'Target levels should be between 0 and 1');
    end
    tgts = tlevs;
end

n_tgts = numel(tgts);
if nargin < 4 || isempty(catnames),
    catnames = make_level_names(fname, n_tgts);
elseif ~(iscellstr(catnames) && length(catnames) == n_tgts),
    error('StimSelect:Sim_Factor:BadCatNames', ...
        'Category names must be cell array of strings to match number of categories');
end

% Make a cell array with names of factor levels in the first row and
% corresponding factor level structures in the second row
levargs(1, :) = catnames;
for i = 1:n_tgts,
    levargs{2, i} = factorlevel(tgts(i));
end

f = factor_def( 'f_itemval_newtrain', item_sim_update_handle, ...
    'f_scale', sim_scalefunc, ...
    levargs{:} );

s = factor(s, fname, f );

end
