function s = testset_sim(test, train, s_func, varargin)
% TESTSET_SIM - Compute similarities between test and training strings
%
% TESTSET_SIM(S, T, F, FPARMS...)
%   Computes average similarity for each test string in cell array S to all
%   the training strings in cell array T.  This can help to
%   determine the characteristics of published stimulus sets.
%
%   F is a function handle identifying the desired similarity function.  It
%   should be a function that takes two arguments S1 and S2, and may also
%   take additional arguments specified in FPARMS.  S1 and S2 may be either
%   strings or cell arrays of strings, and F must be able to handle both.
%   If both are strings, F should return a scalar similarity value.  If
%   either S1 or S2 is a cell array of strings, F should return a matrix of
%   similarity values, with a row for each string in S1 and a column for
%   each string in S2.  The default is @EditSim, which will be called as
%   EditSim(string1, string2, 0.7, 0.7, 1), that is, with insertion and
%   deletion costs of 0.7 and substitution cost of 1 (see Hahn & Bailey,
%   2005).  These default costs can be changed by specifying the FPARMS
%   parameter.
%
%   FPARMS... are optional extra arguments to be passed to F, which will
%   be called as F('string1', 'string2', FPARMS...).  As a special case, if
%   F is not specified, FPARMS defaults to {0.7, 0.7, 1} as described
%   above.  If F is specified but FPARMS is not, FPARMS defaults to {},
%   which passes no extra arguments to F.
%
% SEE ALSO: SIM_FACTOR

% Copyright 2007 Todd M. Bailey
%
% This file is part of AGL StimSelect (AGLSS).
%
%     AGLSS is free software; you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation; either version 3 of the License, or
%     (at your option) any later version.
% 
%     AGLSS is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%   1.1  2007/09/17 13:15:44

if nargin < 3 || isempty(s_func),
    s_func = @EditSim;
    ReplCost = 1;
    IndelCost = 0.7 * ReplCost;
    s_parms = {IndelCost, IndelCost, ReplCost};
elseif nargin < 4,
    s_parms = {};
else
    s_parms = varargin;
end

% Compute mean similarity between potential test items and the new
% training item(s)
s = s_func(test, train, s_parms{:});
s = mean(s, 2);
