function items = item_property(items, varargin)
% ITEM_PROPERTY - Define properties for items in an item set
%
% Syntax
% ITEM_PROPERTY(M, ..., 'pnameX', @F, ...) 
%   Defines a property for each item, named pnameX, and sets its value by
%   calling function F for each item in item set M.  F should be a function
%   that accepts a single input parameter, namely an item structure.  Item
%   structures ordinarily include the following fields (plus any fields
%   defined in previous calls to ITEM_PROPERTY):
%       .name, the character string name of the item. 
%       .gram, 'G' or 'NG', indicating whether the item is grammatical or not. 
%       .factors, which is a placeholder for dynamically updated
%           information about each item to be filled in later
%
% ITEM_PROPERTY(M, ..., 'pnameX', VALS, ...) 
%   Defines a property for each item, named pnameX, and sets its values
%   from the cell array of values VALS, which should be the same length as
%   M.item.
%
% Multiple property-function or property-value pairs can be specified in a
% single call to ITEM_PROPERTY.

% Copyright 2007 Todd M. Bailey
%
% This file is part of AGL StimSelect (AGLSS).
%
%     AGLSS is free software; you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation; either version 3 of the License, or
%     (at your option) any later version.
% 
%     AGLSS is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with this program.  If not, see <http://www.gnu.org/licenses/>.

while ~isempty(varargin),
    pname = varargin{1};
    if ~ischar(pname),
        error('Property name must be a character string');
    end
    if length(varargin) < 2,
        error('Property names and values must be specified in pairs');
    end

    pval = varargin{2};
    if ~iscell(pval) && ~isa(pval, 'function_handle'),
        error('Property value must be a function or a cell array');
    end
    
    varargin(1:2) = [];
    if ~iscell(pval),
        f = pval;
        pval = arrayfun(f, items.item, 'UniformOutput', false);
    end
    % copy cell values to corresponding items
    if numel(pval) ~= length(items.item),
        error('Property value must have one cell for each item');
    end
    [items.item.(pname)] = pval{:};
% %     else
% %         % call function for each item
% %         vals = arrayfun(pval, items.item, 'UniformOutput', false);
% % %         for i = 1:length(items.item),
% % %             items.item(i).(pname) = vals(i);
% % %         end
% %     end
end

end