function [items, isG] = itemset_init(strings, isG)
% itemset_init - Initialize set of potential items for AGL
%
% [I, I_isG] = itemset_init(S, isG)
%  Creates an itemset structure of potential stimulus items.
%   S is a cell array of strings specifying the items to use.
%
%   isG is a logical vector indicating which of S are grammatical (true)
%   and which are ungrammatical (false), so S(isG) will select the
%   grammatical cells, and S(~isG) will select the ungrammatical ones.
%
% The return value I will be a structure containing four fields:
%   I.item will be a structure array with one element for each potential
%   item (in randomised order).  
%
%   I.n_items gives the length of I.item, that is, the number of potential
%   items.
%
%   I.update_func will be an empty cell array for handles to
%   item update functions.  
%
%   I.uptodate_all will be true if all items have been updated to reflect
%   the current training set (which makes them ready to be considered for
%   inclusion in test sets). 
%
% The return value I_isG indicates which of I.item are grammatical (since
% they are placed in a randomised order).
%
% SEE ALSO:
%  fsg, item_sim_update_func, item_property

% Copyright 2007 Todd M. Bailey
%
% This file is part of AGL StimSelect (AGLSS).
%
%     AGLSS is free software; you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation; either version 3 of the License, or
%     (at your option) any later version.
% 
%     AGLSS is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with this program.  If not, see <http://www.gnu.org/licenses/>.

% basic structure for a stimulus item
item_struct = struct(...
    'name', '', ...         % item string
    'gram', '?' ...        % grammatical status, 'G' or 'NG'
    );
    
items = struct(...
    'item', repmat(item_struct, size(strings)), ...
    'n_items', length(strings), ...
    'update_func', {{}}, ... % handles to update functions
    'uptodate_all', true ... % if all items know about training set
    );

[items.item.name] = strings{:};

[items.item(isG).gram] = deal('G');
[items.item(~isG).gram] = deal('NG');

% put items in random order, remembering which are G
k = randperm(items.n_items);
items.item = items.item(k);
isG = isG(k);

end