function m = update_items(m, active_factors, train, newtrain)
% UPDATE_ITEMS - Update items for new training items
%
% UPDATE_ITEMS(ITEMS, FACTORS, TRAIN, NEWTRAIN) 
%   Updates the specified ITEMS (an item structure array) to reflect the
%   addition of one or more new training items NEWTRAIN to the training set
%   TRAIN, for factors FACTORS.
%
% Item values are assumed to be either scalars or row vectors.
%
% The modified items are returned.

% Copyright 2007 Todd M. Bailey
%
% This file is part of AGL StimSelect (AGLSS).
%
%     AGLSS is free software; you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation; either version 3 of the License, or
%     (at your option) any later version.
% 
%     AGLSS is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with this program.  If not, see <http://www.gnu.org/licenses/>.

fnames = fieldnames(active_factors);
for factor_idx = 1:length(fnames),
    fname = fnames{factor_idx};
    curfactor = active_factors.(fname);
    if isfield(curfactor, 'f_itemval_newtrain'),
        f_itemval_newtrain = curfactor.f_itemval_newtrain;
    else
        f_itemval_newtrain = [];
    end
    if ~isempty(f_itemval_newtrain),
        m = f_itemval_newtrain(m, fname, train, newtrain);
    end
    
    if item_isfield(m, fname),
        itemvals = item_val(m, fname);
        itemvals = cat(1, itemvals{:}); % assumes each item val is row vector
        % %     local_processing = false;    % default
        values_scaled = false;      % default
        
        levnames = curfactor.levelnames;
        for level_idx = 1:length(levnames),
            levname = levnames{level_idx};
            curlevel = curfactor.(levname);
            
            if isfield(curlevel, 'tgt'),
                
                if isfield(curlevel, 'f_scale'),
                    f_scale = curlevel.f_scale;
                else
                    f_scale = [];
                end
                if isfield(curlevel, 'f_itemfit'),
                    f_itemfit = curlevel.f_itemfit;
                else
                    f_itemfit = [];
                end
                tgt = curlevel.tgt;
                
                if isempty(f_scale) && ~isempty(f_itemfit)
                    % Make minimal assumptions, just pass the data through
                    % Return value should have one fitness value per item
                    g = f_itemfit(itemvals, tgt, train, newtrain);
                else
                    % %             if ~local_processing,
                    % %                 % item values have not already been converted to a matrix
                    % %                 obs_raw = cat(3,itemvals{:});
                    % %                 local_processing = true;
                    % %             end
                    if isempty(f_scale),
                        g = itemfit_default(itemvals, tgt);
                    else
                        if values_scaled,
                            % item values have been scaled; just do target
                            if iscell(tgt),
                                tgt_scaled = cell(1, length(tgt));
                                empties = cellfun(@isempty, tgt);
                                tgt_scaled(~empties) = num2cell(f_scale([tgt{:}]));
                            else
                                tgt_scaled = f_scale(tgt);
                            end
                        else
                            values_scaled = true;   % to not scale item vals again
                            if iscell(tgt),
                                tgt_scaled = cell(size(tgt));
                                empties = cellfun(@isempty, tgt);
                                tgt_scaled(~empties) = num2cell(f_scale([tgt{:}]));
                                obs_scaled = f_scale(itemvals);
                            else
                                % Transform item values and target to normalized
                                % scale, together for speed
                                x = f_scale([tgt; itemvals(:)]);
                                obs_scaled = x(2:end);
                                tgt_scaled = x(1);
                            end
                        end
                        if isempty(f_itemfit),
                            g = itemfit_default(obs_scaled, tgt_scaled);
                        else
                            g = f_itemfit(obs_scaled, tgt_scaled);
                        end
                    end
                    %             g = reshape(g, size(itemvals));
                end
                
                if isfield(curlevel, 'wt'),
                    wt = curlevel.wt;
                    if wt ~= 1,
                        g = g .* wt;
                    end
                end
                
                % assign one fitness value to each item
                m = item_fit(m, fname, levname, g(:));
            end
        end
    end
end

end


% ITEMFIT_DEFAULT - Compute goodness of fit relative to target value
%
% ITEMFIT_DEFAULT(VALS, TGT) 
%   Computes goodness of fit statistics for values VALS with respect to the
%   target value TGT.
%
% The first dimension of VALS is assumed to correspond to items.  If VALS
% is not a column vector, fitness values are computed for each element of
% VALS, and they are summed across all dimensions but the first.  
%
% Values should be in the range 0-1.
%
% The return value is a column vector whose length = size(VALS, 1).
%
% (c) Todd M. Bailey, 2006
function g = itemfit_default(vals, tgt)

if iscell(tgt),
    % ignore empty cells, just compute fit from non-empty targets along
    % with the corresponding item values.
    empties = cellfun(@isempty, tgt);
    vals = vals(:,~empties);    % ignore value columns with no targets
    siz = size(vals);
    vals = vals(:); % column vector
    change_shape = true;

    tgt = repmat([tgt{~empties}], siz(1), 1);   % Repeat for each item
    tgt = tgt(:);   % column vector
else
    siz = size(vals);

    if length(siz) ~= 2 || siz(2) ~= 1,
        change_shape = true;
        vals = vals(:);
    else
        change_shape = false;
    end

    tgt = tgt .* ones(size(vals));
end

[g_tot, g_mat] = likelihoodchi2([vals 1-vals], [tgt 1-tgt]);
g = sum(g_mat, 2);

if change_shape,
    g = reshape(g, siz(1), []);
    g = sum(g, 2);  % sum up 
end

end