function f = centileScale_func(c25, c50, c75)
% centileScale_func - Create a scaling function
%
% F = centileScale_func(C50)
% F = centileScale_func(C25, C75)
% F = centileScale_func(C25, C50, C75)
% Returns a handle to a function F that scales scores in the range 0-1 so
% that F([0 C25 C50 C75 1]) returns [0 .25 .50 .75 1].  It is required that
% 0 <= C25 <= C50 <= C75 < 1.  F should be called with a single scalar or
% matrix argument, and will return a column vector of the same size, containing
% the scaled values.
%
% Linear interpolation is used.

% Copyright 2007 Todd M. Bailey
%
% This file is part of AGL StimSelect (AGLSS).
%
%     AGLSS is free software; you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation; either version 3 of the License, or
%     (at your option) any later version.
% 
%     AGLSS is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with this program.  If not, see <http://www.gnu.org/licenses/>.

if nargin < 2,
    c50 = c25;
end
if nargin < 3,
    c75 = c50;
end

if c50 < c25 || c75 < c50,
    error('Arguments must increase in magnitude from first to last');
end
if any([c25 c50 c75] <= 0) || any([c25 c50 c75] >= 1),
    error('Arguments must be strictly greater than 0 and less than 1');
end

vals = unique([c25;c50;c75]);
switch length(vals),
    case 1,
        scale = [0.5];
    case 2,
        scale = [0.25; 0.75];
    otherwise,
        scale = [0.25; 0.50; 0.75];
end
vals = [0; vals; 1];
scale = [0; scale; 1];

f = @f1;

    function y = f1(x)
        if isscalar(x),
            % use a faster function for scalars
            y = interp(vals, scale, x);
        else
            y = interp1q(vals, scale, x(:));
        end
    end

end
