function [items, train, test] = choose_items(items, train, n_train, ...
    test, n_test, testfam)
% choose_items - Choose AGL training and/or test items
%
% [I, T, S] = choose_items(I, T, N_T, S, N_S, TF_familiarity)
%  Chooses training items and/or test items for an AGL study from the list
%  of potential items I, according to the target values identified for
%  training set T and the list of test sets S.  
%   N_T specifies the number of training items to be chosen.  
%   N_S specifies the number of test items to be chosen for each test set in S.
%   TF_familiarity specifies whether training items are eligible to also be
%   test items.
%
% I is a structure of potential items (with fields item_struct and update_func).
% T is a training set structure.
% S is a test set structure array.
% N_T and N_S are nonnegative scalars.
%
% T, S, and I are modified appropriately and returned.
%
% If N_T and N_S are both greater than 0, an item is first added to the
% training set, then one item is added to each test set (in random order),
% and so on, until the specified number of items have been added.
%
% The item selection process invokes a number of call-back functions which
% can be specified when factors are defined.  Below is an outline of the
% item selection process:
%   Select one or more new training items
%       - Check per-item training exclusion flags
%       @ Training exclusion call-backs, if any
%       For each available item
%           @ Training update call-backs (provisional new training item)
%           @ Training fitness call-backs (assess provisional new item)
%           @ Test-set update call-backs (temporary)
%       @ Training update call-backs (item with highest fitness above)
%       @ Training update-testsets call-backs
%
%   Select a test item
%       @ Item value new training call-backs (update item values w.r.t. each level of each factor)
%       @ Item value scale call-backs
%       @ Item value fitness call-backs (item contributions to fitness for each level of each factor) 
%       - Check per-item test exclusion flags
%       @ Test exclusion call-backs

% Copyright 2007 Todd M. Bailey
%
% This file is part of AGL StimSelect (AGLSS).
%
%     AGLSS is free software; you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation; either version 3 of the License, or
%     (at your option) any later version.
% 
%     AGLSS is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with this program.  If not, see <http://www.gnu.org/licenses/>.

% save number of items previously chosen
n0_train = train.n_items;
n0_test = test.testsets(1).n_items;

while test.testsets(1).n_items < n0_test+n_test || train.n_items < n0_train+n_train,
    % training item first, if necessary
    if train.n_items < n0_train + n_train,
%         train = update_test_items(train, test);
        
        fprintf(' Choosing training item %d...', train.n_items+1);
        [train, test, items] = add_train_item(items, train, test);
        fprintf('.\n');
    end
    % add test items, if necessary
    if test.testsets(1).n_items < n0_test + n_test,
%         train = update_all_items(train, test);
        train = update_potential_testitems(train, test);

        fprintf(' Choosing test item %d for each set...', ...
            test.testsets(1).n_items+1);
        [test, train, items] = add_test_items(items, test, train, testfam);
        fprintf('.\n');
    end
end
return

    % T = UPDATE_POTENTIAL_TESTITEMS(T, S)
    % This function updates the actual list of ITEMS in the enclosing
    % function.  Because the number of changed items is likely to be large
    % and because they are updated only when actually needed (not just
    % provisionally), they are modified directly instead of being passed in
    % and out.
    function train = update_potential_testitems(train, test)
        if ~items.uptodate_all,
            fprintf(' Updating potential items...');
            to_update = ~testitem_stat(test, items.n_items);
            items.item(to_update) = update_items(...
                items.item(to_update), ...
                test.active_factors, ...
                train, items.item(train.newtrain_pending));
            fprintf('.\n');
        end
        train.newtrain_pending = [];
        items.uptodate_all = true;
    end

end