function s = format_test_items(test, items, detail)
% FORMAT_TEST_ITEMS - Format test items for printout.
%
% FORMAT_TEST_ITEMS(S, I)
% FORMAT_TEST_ITEMS(S, I, 'detail')
% FORMAT_TEST_ITEMS(S, I, 'summary')
%
% If the third argument is 'detail', or is not given, information is
% returned for each item of each test set.
%
% If the third argument is 'summary', information is aggregated across the
% items within each test set.

% Copyright 2007 Todd M. Bailey
%
% This file is part of AGL StimSelect (AGLSS).
%
%     AGLSS is free software; you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation; either version 3 of the License, or
%     (at your option) any later version.
% 
%     AGLSS is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with this program.  If not, see <http://www.gnu.org/licenses/>.

if nargin < 3,
    detail = 'detail';
end
by_item = strcmpi(detail, 'detail');

max_item_len = max(cellfun(@length, {items.item.name}));
tset_num_fmt = '    %02d';
% % tset_gram_fmt = '%2s';
% % tset_fam_fmt = '%3s';
tset_cat_fmt = ' %13s';
item_num_fmt = '      %02d';
item_name_fmt = sprintf(' %%%ds', max_item_len+1);
item_XX_fmt = ' %10.3f';

% Print each item of each tset
s = '';
header_done = false;
for test_idx = 1:test.n_testsets,
    tset = test.testsets(test_idx);
    
    fields = fieldnames(tset);
    cat_cells = regexpi(fields, '_cat$');
    n_cats = length(cat(1, cat_cells{:}));
    
    item_idx = find(tset.items);
    for item_num = 1:tset.n_items,
        if by_item || item_num == tset.n_items,
            [cur_s, cur_h] = headed_data(tset_num_fmt, test_idx, ...
                ' %s', 'Tset_num');
            s = [s, cur_s];
            if ~header_done,
                h = cur_h;
            end
            
            for i = 1:n_cats,
                [cur_s, cur_h] = headed_data(tset_cat_fmt, tset.(fields{i}), ...
                    tset_cat_fmt, fields{i} );
                s = [s, cur_s];
                if ~header_done,
                    h = [h, cur_h];
                end
            end
        end
         
        item = items.item(item_idx(item_num));
           
        if by_item,
            [cur_s, cur_h] = headed_data(item_num_fmt, item_num, ...
                ' %s', ' Itm_num' );
            s = [s, cur_s];
            if ~header_done,
                h = [h, cur_h];
            end
            
            [cur_s, cur_h] = headed_data(item_name_fmt, item.name, ...
                item_name_fmt, ' Itm_name' );
            s = [s, cur_s];
            if ~header_done,
                h = [h, cur_h];
            end
        end
        
        fnames = fieldnames(tset.active_factors);
        n_fnames = length(fnames);
        if item_num == 1,
            vals = cell(1, n_fnames);
            tgts = cell(1, n_fnames);
        end
        for i = 1:n_fnames,
            fname = fnames{i};

            if item_isfield(item, fname),
                v = item_val(item, fname);
                v = v{1};
                if isempty(vals{i}),
                    vals{i} = v;
                else
                    vals{i} = vals{i} + v;
                end
                
                t = testset_factor_field(tset, fname, 'tgt');
                if isempty(tgts{i}),
                    tgts{i} = t;
                else
                    if iscell(t),
                        for cellidx = 1:numel(t),
                            if ~isempty(t{cellidx}),
                                tgts{i}{cellidx} = tgts{i}{cellidx} + ...
                                    t{cellidx};
                            end
                        end
                    else
                        tgts{i} = tgts{i} + t;
                    end
                end
                
                if by_item || item_num == tset.n_items,
                    if ~by_item,
                        v = vals{i} / tset.n_items;
                        if iscell(tgts{i}),
                            for cellidx = 1:numel(tgts{i}),
                                if ~isempty(tgts{i}{cellidx}),
                                    tgts{i}{cellidx} = tgts{i}{cellidx} / ...
                                        tset.n_items;
                                end
                            end
                            t = tgts{i};
                        else
                            t = tgts{i} / tset.n_items;
                        end
                    end
                    [cur_s, cur_h] = headed_data(item_XX_fmt, v, ...
                        ' %10s', fname );
                    s = [s, cur_s];
                    if ~header_done,
                        h = [h, cur_h];
                    end
                    
                    [cur_s, cur_h] = headed_data(item_XX_fmt, t, ...
                        ' %10s', [fname, '_tgt'] );
                    s = [s, cur_s];
                    if ~header_done,
                        h = [h, cur_h];
                        if i == n_fnames,
                            header_done = true;
                        end
                    end
                end
            end
        end
        if by_item || item_num == tset.n_items,
            s = [s, sprintf('\n')];
        end
    end
end
h = [h, sprintf('\n')];
s = [h, s];

end

% Format data, including vectors, and generate appropriate number of
% headers.
function [s, h] = headed_data(data_fmt, data, hdr_fmt, hdr_base);
if ischar(data),
    n = size(data,1);
else
    n = numel(data);
end
if n > 1,
    h = '';
    for i = 1:n,
        h = [h, sprintf(hdr_fmt, ...
            sprintf([hdr_base, '_%d'], i))];
    end
else
    h = sprintf(hdr_fmt, hdr_base);
end
h_width = round(length(h) / n);

if iscell(data),
    for i = 1:numel(data),
        if isempty(data{i}),
            data{i} = NaN;
        end
    end
    data = cat(2, data{:});
end
s = sprintf(data_fmt, data);    % format is used repeatedly for vectors
data_width = round(length(s) / n);

switch sign(data_width - h_width),
    case -1,
        [s, h] = headed_data(...
            [repmat(' ', 1, h_width - data_width), data_fmt], data, ...
            hdr_fmt, hdr_base);
    case +1,
        [s, h] = headed_data(...
            data_fmt, data, ...
            [repmat(' ', 1, data_width - h_width), hdr_fmt], hdr_base);
end

end