function [train, test, items] = add_train_item(items, train, test)
% ADD_TRAIN_ITEM
%
% [T, S, V] = add_train_item(V, T, S)
%
% V is a structure of available items to choose from
% T is a structure array of current training items, with fields
%   T.chfreq (total chunk frequencies within T.items) and T.items 
%   (logical vector same length as V)
% S is a structure array of current test items
%
% Each available item is provisionally added to the training set by calling
% the functions in T.update_func.  The functions in T.fit_func are then
% called to evaluate the goodness of fit of the resulting training set.
% The item that produces a training set with the best fit is then added for
% real, and the resulting training set is returned.  The index of the
% chosen item within V is returned as a second return value.
%
% T.update_func is a cell array of function handles for updating fields in
% T to reflect the addition of a new item.  Each function will be called in
% order as [T, S] = update_func(T, S, I, Titems, Sitems), where T is a
% structure training item set, S is a structure array test item set, I is a
% single new training item from V, Titems is a structure array of item
% structures in the training st and Sitems is a structure array of items in
% the various test sets.  The function should return T and S, modified or
% not.
%
% T.fit_func is a cell array of function handles for computing goodness of
% fit for a training set.  Each function will be called in order as G =
% fit_func(T, S), with training and test sets T and S.  The fit values G
% from all T.fit_func calls are summed up to determine the overall goodness
% of fit for a particular training set.

% Copyright 2007 Todd M. Bailey
%
% This file is part of AGL StimSelect (AGLSS).
%
%     AGLSS is free software; you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation; either version 3 of the License, or
%     (at your option) any later version.
% 
%     AGLSS is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with this program.  If not, see <http://www.gnu.org/licenses/>.

if isempty(train.items),
    train.items = false(1, items.n_items);
end

test_items = testitem_stat(test, items.n_items);
any_test_items = any(test_items);

if isempty(train.excluded_items),
    train.excluded_items = false(1, items.n_items);
end

cur_avail = ~train.items & ~train.excluded_items & ~test_items;
[exclude, train] = check_training_exclusions(train, cur_avail);
train.excluded_items = train.excluded_items | exclude;

avail = find(~train.items & ~train.excluded_items & ~test_items);

n_avail = length(avail);
if n_avail < 1,
    error('No items available');
end

train_item_struct = items.item(train.items);

fits = zeros(n_avail,1);
for item_idx = 1:n_avail,
    % Try each available item in turn
    cur_item = items.item(avail(item_idx));
    cur_train = train;
    % Call each training update function in turn with the current new item
    for upd_idx = 1:length(train.update_func),
        cur_train = train.update_func{upd_idx}( ...
            cur_train, train_item_struct, cur_item);
    end
    cur_train.items(item_idx) = true;
    cur_train.n_items = train.n_items + 1;

    % Compute fits for training items
    for upd_idx = 1:length(train.fit_func),
        fits(item_idx) = fits(item_idx) + ...
            train.fit_func{upd_idx}(cur_train);
    end

    % Tell each set of test items about the tentative new training item
    if any_test_items,
        cur_test = update_testsets(test, cur_train, cur_item, 'temp');
        fits(item_idx) = fits(item_idx) + testfit(cur_test);
    end
end
[min_fit, avail_idx] = min(fits);
idx = avail(avail_idx);
new_item = items.item(idx);

% add the best item to the training set
for upd_idx = 1:length(train.update_func),
    train = train.update_func{upd_idx}(train, test, new_item);
end
train.items(idx) = true;
train.n_items = train.n_items + 1;

if any_test_items,
    [test, updated_items, i_idx] = ...
        update_testsets(test, train, new_item, 'full');
    [items.item(i_idx)] = updated_items;
end

items.uptodate_all = false;
train.newtrain_pending(end+1) = idx;

return

    % UPDATE_TESTSETS - Tell testsets and their items about new train items
    %
    % This function is nested so that it can refer to (and modify) ITEMS in
    % the enclosing function, avoiding the cost of passing in this large
    % data structure on the many calls to UPDATE_TESTSETS (to assess the
    % potential impact of each candidate training item).
    function [test, updated_items, idx] = ...
            update_testsets(test, train, new_item, flag)
        updated_items = [];
        idx = [];
        
        % Tell each set of test items about the new training item
        for test_idx = 1:test.n_testsets,
            cur_testset = test.testsets(test_idx);
            testset_item_idx = find(cur_testset.items);

            cur_updated_items = update_items(...
                items.item(testset_item_idx), ...
                cur_testset.active_factors, ...
                train, new_item );
            updated_items = [updated_items; cur_updated_items];
            idx = [idx, testset_item_idx];

            cur_testset = testset_newtrain(cur_testset, ...
                cur_updated_items, flag);
            test.testsets(test_idx) = cur_testset;
        end
    end

    function [exclude, train] = check_training_exclusions(train, avail)
        % use items and test from enclosing function

        exclude = false(1, items.n_items);
        new_tx = train.exclusion_task([]);  % make empty list of same type
        for x_idx = 1:length(train.exclusion_task),
            cur_x = train.exclusion_task(x_idx);    % get function handle or structure
            if isstruct(cur_x),
                % not a raw function handle; get the handle from inside the
                % struct
                cur_x = cur_x.func;
            end
            still_avail = find(avail & ~exclude);
            
            [xn, reschedule] = cur_x(items.item(still_avail), train, test);
            
%             reschedule = false;
%             cur_exclude = false(1,items.n_items);
%             for i = 1:length(still_avail),
%                 item_idx = still_avail(i);
%                 [cur_exclude(item_idx), r] = ...
%                     cur_x.func(items.item(item_idx), train, test);
%                 reschedule = reschedule | r;
%             end

            if reschedule,
                new_tx(end+1) = cur_x;
            end
            
            if length(xn) == length(exclude),
                % add relevant subset to excluded ones
                exclude(still_avail) = exclude(still_avail) | xn(still_avail);
            else
                % f_exclude must have focused on still_avail subset
                exclude(still_avail) = exclude(still_avail) | xn;
            end
            
%             exclude(still_avail) = exclude(still_avail) | xn;
        end
        train.exclusion_task = new_tx;
    end

end

