function [tgt_f, wt] = chunk_target_freqs(freq, f_cats, f_cat_wts, f_cat_sizes)
% ASSIGN_CHUNK_FREQS - Assign chunks to categories of various target
% frequencies
%
% [T, W] = chunk_target_freqs(F)
%
% [T, W] = chunk_target_freqs(F, TCAT)
%
% [T, W] = chunk_target_freqs(F, TCAT, WCAT)
%
% [T, W] = chunk_target_freqs(F, TCAT, WCAT, NCAT)
%
% Returns target frequencies T and weightings W for the chunks whose
% frequencies among all grammatical items are given in F.  The target
% frequency for each chunk is one of the values given in vector TCAT.  The
% relative importance of achieving that target frequency (the weighting)
% comes from the corresponding value in vector WCAT.  If the category size
% vector NCAT is specified, it determines the absolute or relative number
% of chunks allocated to each frequency category.
%
% Nonnegative values in TCAT specify relative frequencies, so that [1 3]
% means that chunks of one category should appear 3 times as often as the
% other.  Exceptionally, negative values in TCAT specify absolute target
% frequencies.  The magnitude of the negative value specifies the target 
% frequency.  Unless you know in advance the total number of chunks, TCAT
% should normally include at least one value greater than 0 (i.e. one
% relative frequency).
%
% The values in TCAT should be arranged in ascending order of absolute
% frequency.  In particular, to determine the appropriate position for
% absolute target frequencies, it is necessary to estimate the final total
% number of chunks and compute the final frequency of chunks in the various
% categories of relative frequencies.  Consider TCAT = [0 -1 1 3],
% specifying a category of unseen chunks, a category of chunks which appear
% just once, and two additional categories, one of which has chunks which
% appear 3 times as often as the other.  We do not necessarily know the
% absolute frequency of chunks in the low relative frequency category, but
% they should generally appear more than once so it is appropriate to
% specify the relative frequency 1 after the absolute frequency -1 in TCAT.
% On the other hand, if we wanted a category of chunks to appear exactly 10
% times, we would have to decide whether [0 -10 1 3] or [0 1 -10 3] or [0 1
% 3 -10] was most appropriate, based on the total number of chunks
% expected.
%
% If TCAT is not given, or is [], then all chunks are assigned the same
% target frequency.
%
% If WCAT is not given, or is [], then all frequency categories are given
% weighting 1.
%
% If NCAT is omitted or [], the categories have approximately equal number
% of chunks, except that if TCAT includes the value 0 (so that some chunks
% should be unseen during training), then as a minimum all chunks X whose
% frequency F[X] == 0 are assigned to the category with target frequency 0.
%
% If the vector NCAT is given, it should have the same length as F.
% Positive values specify relative numbers of chunks, so that [1 3] means that
% the first category should have 3 times as many chunks as the second.
% Negative values in NCAT specify absolute numbers of chunks.  The
% magnitude of the negative value specifies the number of chunks.  

% Copyright 2007 Todd M. Bailey
%
% This file is part of AGL StimSelect (AGLSS).
%
%     AGLSS is free software; you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation; either version 3 of the License, or
%     (at your option) any later version.
% 
%     AGLSS is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with this program.  If not, see <http://www.gnu.org/licenses/>.

if nargin < 4,
    f_cat_sizes = [];
    if nargin < 3,
        f_cat_wts = [];
        if nargin < 2,
            f_cats = 1;
        end
    end
end

% determine size of chunks, assuming freq has nsyms^chsize elements
n_syms = length(freq);  % number of symbols from which chunks are composed
n_chunks = numel(freq);  % total number of chunks to choose from
chsize = round(log(n_chunks) / log(n_syms));    % round off just in case

n_cats = length(f_cats);

% allocate space for return values
tgt_f = ones(1, n_chunks);
wt = ones(1, n_chunks);

n_zero = sum(freq(:) == 0);
% % if n_zero,
% %     % report number of chunks that won't appear in training items, either
% %     % because they are ungrammatical or they have been reserved to be used
% %     % as novel chunks in test items
% %     fprintf(1,'\t%d of %d chunks of length %d will not appear in training items\n', ...
% %         n_zero, n_chunks, chsize);
% % end

if n_cats <= 1,
%     tgt_f = ones(1, n_chunks);
%     wt = ones(1, n_chunks);
    return
end

if isempty(f_cat_wts),
    f_cat_wts = ones(1, n_cats);
elseif length(f_cat_wts) ~= n_cats,
    error('Vector of weights and frequency categories must be same length');
end

if isempty(f_cat_sizes),
    f_cat_sizes = ones(1, n_cats);
elseif length(f_cat_sizes) ~= n_cats,
    error('Vector of category sizes and frequency categories must be same length');
end

% sort chunk frequencies in ascending order
[freq_sorted, freq_idx] = sort(reshape(freq,1,[]));
% NB: chunks{XX}(freq_idx,:) lists chunks in frequency order

isabs = f_cat_sizes < 0;
x_abs = find(isabs);
n_abs = length(x_abs);
t_abs = sum(-f_cat_sizes(x_abs));
if t_abs > n_chunks,
    error('Total absolute chunks (%d) exceeds number of chunks of this size (%d)', ...
        x_abs, n_chunks);
end

cur_n_chunks = n_chunks - t_abs; % number of chunks left for relative cats 
cur_n_cats = sum(f_cat_sizes(~isabs)); % n weighted rel cats not done
cur_n_assigned = 0;
for cidx = 1:n_cats,
    sz = f_cat_sizes(cidx);
    if sz < 0,
        % Assign an absolute number of chunks to this category
        cursize = -sz;
        if f_cats(cidx) == 0 && n_zero > cursize,
            warning(...
                'Only %d of %d unseen chunks assigned to zero-frequency category', ...
                cursize, n_zero);
        end
    else
        % Assign a relative number of chunks to this category
        if cidx == 1 && f_cats(1) == 0,
            % allocate unseen chunks
            cursize = max( n_zero, floor(cur_n_chunks * sz / cur_n_cats) );
        else
            cursize = ceil(cur_n_chunks * sz / cur_n_cats);
        end
        cur_n_chunks = cur_n_chunks - cursize;
        cur_n_cats = cur_n_cats - 1;
    end

% %     if f_cats(cidx) == 0,
% %         fprintf(1,'\t%d of %d chunks of length %d will not appear in training items\n', ...
% %             cursize, n_chunks, chsize);
% %     end

    cat_idx = freq_idx(cur_n_assigned + 1 : cur_n_assigned + cursize);
    tgt_f(cat_idx) = f_cats(cidx);
    wt(cat_idx) = f_cat_wts(cidx);
    
    cur_n_assigned = cur_n_assigned + cursize;
end

end