function f = fwt(chfreq_g, tgt_chstr, n_novel, novel_pmax)
% FWT - Create target chunk frequency structure
%
% FWT(G, CHSTR, NOV, PMAX)
%   Returns a target chunk frequency structure suitable for passing to
%   TRAINSET_INIT().
%
%   G is a cell array, with G{i} being a matrix of frequencies for chunks
%   of size i relative to the grammatical items. 
%
%   CHSTR is a cell array with one cell for each different chunk size of
%   interest, specifying target chunk strength categories for each chunk
%   size. Each cell should be a vector of relative chunk strengths.  For
%   example, [1 3] specifies two categories, one of which has average chunk
%   strength three times as high as the other.  The vector can be [] or a
%   scalar to specify a single chunk strength category.
%
%   NOV is a vector the same length as CHSTR, specifying the minimum
%   number of novel chunks of each size, that is, the minimum number of
%   chunks that should be avoided in training items over and above any
%   chunks that do not appear in any sampled grammatical items.
%
%   PMAX is a vector the same length as CHSTR, specifying the maximum
%   fraction of novel chunks per item to be compensated for, to maintain
%   the average chunk frequency.  PMAX(i) specifies the maximum fraction
%   for chunks of size I.

% Copyright 2007 Todd M. Bailey
%
% This file is part of AGL StimSelect (AGLSS).
%
%     AGLSS is free software; you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation; either version 3 of the License, or
%     (at your option) any later version.
% 
%     AGLSS is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with this program.  If not, see <http://www.gnu.org/licenses/>.

% % g_stat = char(items.item.gram);
% % isG = g_stat(:, 1) == 'G';
% % strings_g = {items.item(isG).name};
% % chfreq_g = ngram(strings_g, syms, 1:ngram_max);

f = []; % Start with empty frequency-weight structure
for idx = 1:length(tgt_chstr),
    tgt_freqs = unique( tgt_chstr{idx} );   % also sorts, ascending order
    if isempty(tgt_freqs),
        tgt_freqs = [1];    % one frequency category, all chunks the same
    end

    if isempty(n_novel),
        n_nov = 0;
    else
        n_nov = n_novel(idx);
    end
    
    ch_freqs = chfreq_g{idx};
    
    % create some additional target frequency categories that can be
    % combined with unseen chunks to produce overall average frequencies
    % corresponding to original frequency targets.  Assume that the
    % unseen chunk is replacing what would otherwise have been a low
    % frequency chunk, not a high frequency chunk.
    if novel_pmax(idx) > 0,
        comp_freqs = tgt_freqs + tgt_freqs(1) * ...
            novel_pmax(idx) / (1 - novel_pmax(idx));
        for i = length(comp_freqs)-1:-1:1,
            if comp_freqs(i) > mean([tgt_freqs(i) tgt_freqs(i+1)]),
                % do not bother creating a new chunk frequency category
                % if it would be close to or bigger than next higher
                % category anyway
                comp_freqs(i) = [];
            end
        end
        tgt_freqs = sort([tgt_freqs, comp_freqs]);
    end

% %         tgt_freqs = unique([0, tgt_freqs, 1.5 .* tgt_freqs]);

    % how many chunks appear in no grammatical items?
    n_z = sum(ch_freqs(:) == 0);
    n_cats = length(tgt_freqs); % number of frequency categories
    n_chpc = floor(numel(ch_freqs) / n_cats); % chunks per cat

    if n_z > 0 || n_nov > 0,
        tgt_freqs = unique([0, tgt_freqs]);
% %         if n_chpc - n_z < n_nov,
        % exclude NG chunks from assignment to normal frequency categories,
        % and make extra chunks be unseen if necessary, so that grammatical
        % test items can include unseen (novel) chunks
        cat_sizes = [-(n_z+n_nov) ones(1, n_cats)];
% %         end
    else
        cat_sizes = []; % equal-sized categories
    end

    [f(idx).freq, f(idx).wt] = ...
        chunk_target_freqs(ch_freqs, tgt_freqs, ...
        [], cat_sizes);
    f(idx).n_chunks = numel(f(idx).freq);
    f(idx).n_tgt0 = sum(f(idx).freq(:) == 0);

    if idx < length(tgt_chstr),
        % force bigger chunks containing unseen chunks to be allocated as
        % unseen
        z = f(idx).freq(:) == 0;
        bigger_freq = chfreq_g{idx+1};
        bigger_freq(:, z) = 0;
        porder = [idx+1 1:idx];
        bigger_freq = permute(bigger_freq, porder);
        bigger_freq(:,z) = 0;
        chfreq_g{idx+1} = ipermute(bigger_freq, porder);
    end
end

end
