function [f, syms, chunks] = trainset_chfreq(train, max_chsize, syms)
% TRAINSET_CHFREQ - Compute frequency of chunks in training items
%
% F = TRAINSET_CHFREQ(T, SIZE)
% F = TRAINSET_CHFREQ(T, SIZE, SYMS)
% [F, SYMS] = TRAINSET_CHFREQ(...)
% [F, SYMS, CX] = TRAINSET_CHFREQ(...)
%   Intended to help determine the characteristics of published stimuli.
%   Returns a cell array of matrices indicating the frequency of chunks of
%   size 1 to SIZE in the set of training items T.  F{i} will be an
%   i-dimensional matrix M in which element M(x1, ..., xi) indicates the
%   number of times the chunk string SYMS([x1, ..., xi]) appears in T.
%
%   T can be a cell array of strings or a character array.
%
%   SIZE should be a positive integer indicating the maximum chunk size of
%   interest.
%
%   SYMS is an optional string indicating the characters (symbols) of
%   interest.  The strings in T should be composed only of these
%   characters.  If SYMS is not specified, it is computed from the
%   characters contained in T.
%
% [F, SYMS] = TRAINSET_CHFREQ(...) also returns a string listing the
% unique characters contained in T.
%
% [F, SYMS, CX] = TRAINSET_CHFREQ(...) returns a cell array, with each cell
% i containing a character array listing the possible chunks of size i.
% F{i}(j) gives the frequency of the chunk CX{i}(j,:) in T.
%
% SEE ALSO: chunks_init

% Copyright 2007 Todd M. Bailey
%
% This file is part of AGL StimSelect (AGLSS).
%
%     AGLSS is free software; you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation; either version 3 of the License, or
%     (at your option) any later version.
% 
%     AGLSS is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%   1.1  2007/09/17 10:43:24

if nargin < 2 || isempty(max_chsize),    
    max_chsize = 2;
end

if nargin < 3 || isempty(syms),
    syms = unique_chars(train);
end

if nargout > 1,
    chunks = chunk_names(syms, max_chsize);
end

f = ngram(train, syms, 1:max_chsize);

end
