function t = trainset_init(items, isavail, freq_wt_tgt);
% TRAINSET_INIT - Initialize a set of AGL training items.
%
% TRAINSET_INIT(I, A, F) returns an empty AGL training set structure
% which will be prepared to choose items from the subset of the item
% structure array I indicated by the logical vector of available items A,
% subject to weighted chunk frequency targets F.  
%
% A should have the same length as I.
%
% The training set structure will include exclusion constraints for
% absolute frequency targets (those with nonpositive frequency values
% specified in F.freq -- see CHUNK_TARGET_FREQS).  These constraints will
% exclude items that contain chunks that are meant to be unseen during
% training (target frequencies of zero).  Also, items that contain chunks
% that are meant to occur a specified number of times will be excluded
% once the target number of occurrences has been achieved.

% Copyright 2007 Todd M. Bailey
%
% This file is part of AGL StimSelect (AGLSS).
%
%     AGLSS is free software; you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation; either version 3 of the License, or
%     (at your option) any later version.
% 
%     AGLSS is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with this program.  If not, see <http://www.gnu.org/licenses/>.

% empty training set
t = struct(...
    'items', [], ...         % logical vector indicating training items
    'excluded_items', [], ... % logical vector indicating disqualified items
    'n_items', 0, ...        % number of items
    'fit', NaN, ...          % overall goodness of fit
    'update_func', {{}}, ... % handles to update functions
    'fit_func', {{}}, ...     % handles to goodness of fit functions
    'exclusion_task', [], ...  % list of tasks to find ineligible items
    'newtrain_pending', [] ... % list of new training items updated in test items only
    );

if isfield(items,'item'),
    items = items.item;
end

if length(isavail) ~= length(items),
    error('Length of available item flags must match number of items');
end

t.items = false(1,length(items));   % initially no items in training set

t.excluded_items = reshape(~isavail, 1, []); % make sure it's a row vector

% add fields for chunk frequencies and their target values
if ~isempty(freq_wt_tgt),
    t = add_field(t, 'freq_tgt', freq_wt_tgt);
    t = add_field(t, 'n_chunks_avail', ...
        cat(2, freq_wt_tgt.n_chunks) - cat(2, freq_wt_tgt.n_tgt0));
    n_sizes = length(freq_wt_tgt);
    t = add_field(t, 'chfreq', cell(1, n_sizes), ...
        @train_chfreq_update, @train_chfreq_fit);
    % % % add field for chunk strength categories (see FWT and CHSTR_FACTOR)
    % % t = add_field(t, 'tgt_freqs', tgt_freqs);

    % Exclude items that have chunks targeted for zero frequency.
    for size_idx = 1:n_sizes,
        cur_f = freq_wt_tgt(size_idx).freq;
        chunk_idx = find(cur_f == 0);
        if ~isempty(chunk_idx),
            for i = find(isavail),
                t.excluded_items(i) = t.excluded_items(i) | ...
                    any(items(i).chfreq{size_idx}(chunk_idx) > 0);
            end
        end
    end

    % Exclude items that have chunks targeted for a fixed absolute frequency,
    % once they have achieved that frequency.
    %
    %   Note that to avoid overshooting absolute frequency targets it is
    %   necessary to penalize overshoot in the relevant fit function....
    for size_idx = 1:n_sizes,
        cur_f = freq_wt_tgt(size_idx).freq;
        chunk_idx = find(cur_f < 0);
        if ~isempty(chunk_idx),
            freq_tgts = -cur_f(chunk_idx);   % absolute target frequency
            t = add_train_exclusion(t, 'abs freq check', ...
                item_has_unwanted_chunk_func(size_idx, chunk_idx, freq_tgts) );
        end
    end
end

    function handle = item_has_unwanted_chunk_func(size_idx, chunk_idx, freq_tgts)
        handle = @item_has_unwanted_chunk;
        
        % Embedded function to check whether a particular chunk has achieved
        % its absolute target frequency, and if so, exclude items containing
        % that chunk from further consideration as potential training items.
        % See ADD_TRAIN_EXCLUSION.
        %
        % The function is embedded so that it can refer to the values of
        % SIZE_IDX, CHUNK_IDX, and FREQ_TGTS in the embedding function, with the
        % values they had when the function handle was created.
        function [exclude, reschedule] = item_has_unwanted_chunk(item, train, test)
            % does this chunk appear enough already in the training set?
            if isempty(train.chfreq{size_idx}),
                train_freqs = 0*chunk_idx;
            else
                train_freqs = train.chfreq{size_idx}(chunk_idx);
            end
            tgt_achieved = train_freqs >= freq_tgts;

            % if so, is it contained in the item in question?
            exclude = any(tgt_achieved & (item.chfreq{size_idx}(chunk_idx) > 0));

            % if chunk has not appeared enough times yet, check again on the
            % next round
            reschedule = ~all(tgt_achieved);
        end
    end

end